from django.db import models

from django_cradmin import datetimeutils
from django_cradmin.viewhelpers.listfilter.basefilters.single import abstractselect
from django_cradmin.viewhelpers.listfilter.django.base import DjangoOrmFilterMixin


class Boolean(abstractselect.AbstractBoolean, DjangoOrmFilterMixin):
    """
    A boolean filter that works on any BooleanField.
    False and None is considered ``False``.

    Examples:

        A model for this example::

            class MyModel(models.Model):
                is_active = models.BooleanField(default=True)

        Create the filter::

            listfilter.django.single.select.Boolean(
                slug='is_active', label='Is active?')
    """
    def get_query(self, modelfield):
        return (models.Q(**{modelfield: False}) |
                models.Q(**{'{}__isnull'.format(modelfield): True}))

    def filter(self, queryobject):
        modelfield = self.get_modelfield()
        cleaned_value = self.get_cleaned_value()
        query = self.get_query(modelfield)
        if cleaned_value == 'true':
            queryobject = queryobject.exclude(query)
        elif cleaned_value == 'false':
            queryobject = queryobject.filter(query)
        return queryobject


class IsNotNull(Boolean, DjangoOrmFilterMixin):
    """
    A subclass of :class:`.Boolean` that works with
    foreign keys and other fields where ``None`` means no
    value and anything else means that it has a value.

    Examples:

        A model for this example::

            class MyModel(models.Model):
                owner = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True)

        Create the filter::

            listfilter.django.single.select.IsNotNull(
                slug='owner', label='Has owner?')
    """
    def get_query(self, modelfield):
        return models.Q(**{'{}__isnull'.format(modelfield): True})


class DateTime(abstractselect.AbstractDateTime, DjangoOrmFilterMixin):
    """
    A datetime filter that works with Django DateTimeField.

    Examples:

        A model for this example::

            class MyModel(models.Model):
                created_datetime = models.DateTimeField(auto_now_add=True)

        Create the filter::

            listfilter.django.single.select.DateTime(
                slug='created_datetime', label='Created time')
    """
    def filter_datetime_range(self, queryobject, start_datetime, end_datetime):
        modelfield = self.get_modelfield()
        start_datetime = datetimeutils.make_aware_in_default_timezone(start_datetime)
        end_datetime = datetimeutils.make_aware_in_default_timezone(end_datetime)
        return queryobject.filter(**{
            '{}__gte'.format(modelfield): start_datetime,
            '{}__lt'.format(modelfield): end_datetime,
        })


class NullDateTime(DateTime):
    """
    Just like :class:`.DateTime`, but adds an option for ``None``-values.

    Examples:

        A model for this example::

            class MyModel(models.Model):
                banned_datetime = models.DateTimeField(null=True, blank=True, default=None)

        Create the filter::

            listfilter.django.single.select.NullDateTime(
                slug='banned_datetime', label='Banned time')
    """
    def null_enabled(self):
        return True

    def filter_is_null(self, queryobject):
        modelfield = self.get_modelfield()
        return queryobject.filter(**{
            '{}__isnull'.format(modelfield): True
        })

    def filter_is_not_null(self, queryobject):
        modelfield = self.get_modelfield()
        return queryobject.filter(**{
            '{}__isnull'.format(modelfield): False
        })


class AbstractOrderBy(abstractselect.AbstractOrderBy, DjangoOrmFilterMixin):
    """
    A "filter" that lets the user select how the queryobject should be ordered.

    You only have to override the ``get_ordering_options()``-method from
    :meth:`~django_cradmin.viewhelpers.listfilter.basefilters.single.abstractselect.AbstractOrderBy`.

    Examples:

        A model for this example::

            class Person(models.Model):
                name = models.CharField(max_length=255)

        Create a subclass of AbstractOrderBy::

            class OrderPersonsFilter(listfilter.django.single.select.AbstractOrderBy):
                def get_ordering_options(self):
                    return [
                        ('name', {
                            'label': 'Name',
                            'order_by': ['name'],
                        }),
                        ('name (descending)', {
                            'label': 'Name (descending)',
                            'order_by': ['-name'],
                        }),
                    ]

        And create the filter::

            OrderPersonsFilter(slug='orderby', label='Order by')
    """
    def filter(self, queryobject):
        cleaned_value = self.get_cleaned_value() or ''
        if cleaned_value in self.ordering_options_dict:
            order_by = self.ordering_options_dict[cleaned_value]['order_by']
            queryobject = queryobject.order_by(*order_by)
        return queryobject



import warnings

from django.template.loader import render_to_string

from django_cradmin import crsettings


def join_css_classes_list(css_classes_list):
    """
    Join the provided list of css classes into a string.
    """
    return '  '.join(css_classes_list)


class AbstractRenderable(object):
    """
    An abstract class that implements an interface for
    rendering something.

    Everything is just helpers for the :meth:`.render` method,
    which renders a template with an object of this class as
    input.
    """

    #: The default value for :meth:`.get_template_names`.
    template_name = None

    def get_template_names(self):
        """
        Get the template name(s) for :meth:`.render`.

        Defaults to :obj:`~.AbstractRenderable.template_name`.

        Raises:
            NotImplementedError: If :obj:`~.AbstractRenderable.template_name` is
             not set.
        """
        if self.template_name:
            return self.template_name
        else:
            raise NotImplementedError('You must set template_name or override '
                                      'get_template_names().')

    def get_context_data(self, request=None):
        """
        Get context data for :meth:`.render`.

        Defaults to::

            {
                'me': self
            }
        """
        return {
            'me': self
        }

    def render(self, request=None, extra_context_data=None):
        """
        Render :obj:`.get_template_names` with
        the context returned by :meth:`.get_context_data`.

        Paramteters:
            request (HttpRequest): If this is provided, we forward it to
                :meth:`.get_context_data`, and to ``render_to_string()``
                (which is used to render the template).
        """
        context_data = {}
        if extra_context_data:
            context_data.update(extra_context_data)
        context_data.update(self.get_context_data(request=request))
        return render_to_string(
            template_name=self.get_template_names(),
            context=context_data,
            request=request)


class AbstractRenderableWithCss(AbstractRenderable):
    """
    Extends :class:`.AbstractRenderable` with a unified
    API for setting CSS classes.
    """
    def get_base_css_classes_list(self):
        return []

    def get_extra_css_classes_list(self):
        return []

    def get_css_classes_list(self):
        """
        Override this to define css classes for the component.

        Must return a list of css classes.

        See :meth:`.get_css_classes_string`.
        """
        css_classes_list = []
        # if hasattr(self, 'get_base_css_classes_list'):
        #     warnings.warn("AbstractRenderableWithCss.get_base_css_classes_list() is deprectated "
        #                   "- override get_css_classes_list() instead.",
        #                   DeprecationWarning)
        #     css_classes_list.extend(self.get_base_css_classes_list())
        # if hasattr(self, 'get_extra_css_classes_list'):
        #     warnings.warn("AbstractRenderableWithCss.get_extra_css_classes_list() is deprectated "
        #                   "- override get_css_classes_list() instead.",
        #                   DeprecationWarning)
        #     css_classes_list.extend(self.get_extra_css_classes_list())
        css_classes_list.extend(self.get_base_css_classes_list())
        css_classes_list.extend(self.get_extra_css_classes_list())
        return css_classes_list

    def get_test_css_class_suffixes_list(self):
        """
        List of css class suffixes to include when running automatic tests.

        These suffixes are filtered through the
        :func:`~django_cradmin.templatetags.cradmin_tags.cradmin_test_css_class`
        template tag.
        """
        return []

    @property
    def css_classes(self):
        """
        Get css classes.

        Joins :meth:`.get_css_classes_list` into a string.

        You should not override this, override :meth:`.get_css_classes_list` instead.
        """
        from django_cradmin.templatetags import cradmin_tags  # Avoid circular import
        css_classes = list(self.get_css_classes_list())
        if crsettings.get_setting('DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES', False):
            for css_class_suffix in self.get_test_css_class_suffixes_list():
                css_classes.append(cradmin_tags.cradmin_test_css_class(css_class_suffix))
        return join_css_classes_list(css_classes)

    def get_css_classes_string(self):
        warnings.warn("AbstractRenderableWithCss.get_css_classes_string() is deprectated "
                      "- use the AbstractRenderableWithCss.css_classes property instead.",
                      DeprecationWarning)
        return self.css_classes



from __future__ import unicode_literals

import json

import django_cradmin
from django import template
from django.conf import settings
from django.contrib.staticfiles.templatetags.staticfiles import static
import warnings
from django.urls import reverse

from django_cradmin import crapp
from django_cradmin import crsettings
from django_cradmin.crinstance import reverse_cradmin_url

register = template.Library()


@register.simple_tag(takes_context=True)
def cradmin_titletext_for_role(context, role):
    """
    Template tag implementation of
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.get_titletext_for_role`.
    """
    request = context['request']
    cradmin_instance = request.cradmin_instance
    return cradmin_instance.get_titletext_for_role(role)


@register.assignment_tag(takes_context=True)
def cradmin_descriptionhtml_for_role(context, role):
    """
    Template tag implementation of
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.get_titletext_for_role`.
    """
    request = context['request']
    cradmin_instance = request.cradmin_instance
    return cradmin_instance.get_descriptionhtml_for_role(role)


@register.simple_tag(takes_context=True)
def cradmin_rolefrontpage_url(context, role):
    """
    Template tag implementation of
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.rolefrontpage_url`.
    """
    request = context['request']
    cradmin_instance = request.cradmin_instance
    return cradmin_instance.rolefrontpage_url(cradmin_instance.get_roleid(role))


@register.simple_tag(takes_context=True)
def cradmin_appurl(context, viewname, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crapp.App.reverse_appurl`.

    Examples:

        Reverse the view named ``"edit"`` within the current app:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_appurl "edit" %}'>Edit</a>

        Reverse a view with keyword arguments:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_appurl "list" mode="advanced" orderby="name" %}'>
                Show advanced listing ordered by name
            </a>
    """
    request = context['request']
    return request.cradmin_app.reverse_appurl(viewname, args=args, kwargs=kwargs)


@register.simple_tag(takes_context=True)
def cradmin_appindex_url(context, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.BaseCrAdminInstance.appindex_url`.

    Examples:

        Reverse index (frontpage) of current app:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_appindex_url %}'>
                Go to pages-app
            </a>

        Reverse a view with keyword arguments:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_appindex_url mode="advanced" orderby="name" %}'>
                Show advanced listing ordered by name
            </a>
    """
    request = context['request']
    return request.cradmin_app.reverse_appurl(
        viewname=crapp.INDEXVIEW_NAME, args=args, kwargs=kwargs)


@register.simple_tag(takes_context=True)
def cradmin_instance_appindex_url(context, appname, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.BaseCrAdminInstance.appindex_url`.

    Examples:

        Reverse index (frontpage) of the ``"pages"`` app:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_instance_appindex_url appname="pages" %}'>
                Go to pages-app
            </a>

        Reverse a view with keyword arguments:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_instance_appindex_url appname="pages" mode="advanced" orderby="name" %}'>
                Show advanced listing ordered by name
            </a>
    """
    request = context['request']
    return request.cradmin_instance.reverse_url(
        appname=appname, viewname=crapp.INDEXVIEW_NAME, args=args, kwargs=kwargs)


@register.simple_tag(takes_context=True)
def cradmin_instanceindex_url(context, appname):
    warnings.warn("cradmin_instanceindex_url is deprecated. Use cradmin_instance_appindex_url instead.",
                  DeprecationWarning)
    return cradmin_instance_appindex_url(context, appname)


@register.simple_tag(takes_context=True)
def cradmin_instance_url(context, appname, viewname, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.BaseCrAdminInstance.reverse_url`.

    Examples:

        Reverse the view named ``"edit"`` within the app named ``"pages"``:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_instance_url appname="pages" viewname="edit" %}'>
                Edit
            </a>

        Reverse a view with keyword arguments:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_instance_url appname="pages" viewname="list" mode="advanced" orderby="name" %}'>
                Show advanced pages listing ordered by name
            </a>
    """
    request = context['request']
    return request.cradmin_instance.reverse_url(
        appname=appname, viewname=viewname, args=args, kwargs=kwargs)


@register.simple_tag
def cradmin_instanceroot_url(instanceid):
    """
    Get the URL of the cradmin instance with the provided ``instanceid``.

    Args:
        instanceid: The :obj:`~django_cradmin.crinstance.BaseCrAdminInstance.id`
            if a :class:`django_cradmin.crinstance.BaseCrAdminInstance`.
    """
    return reverse(instanceid)


@register.simple_tag(takes_context=True)
def cradmin_instance_rolefrontpage_url(context):
    request = context['request']
    return request.cradmin_instance.rolefrontpage_url()


@register.simple_tag(takes_context=True)
def cradmin_url(context, instanceid=None, appname=None, roleid=None, viewname=crapp.INDEXVIEW_NAME, *args, **kwargs):
    """
    Template tag implementation of :meth:`django_cradmin.crinstance.reverse_cradmin_url`.

    Examples:

        Reverse the view named ``"edit"`` within the app named ``"pages"`` in the
        cradmin-instance with id ``"my_cradmin_instance"`` using roleid ``10``:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_instance_url instanceid="my_cradmin_instance" appname="pages"
            roleid=10 viewname="edit" %}'>
                Edit
            </a>

        Reverse a view with keyword arguments:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            <a href='{% cradmin_instance_url instanceid="my_cradmin_instance" appname="pages"
            roleid=10 viewname="list" mode="advanced" orderby="name" %}'>
                Show advanced pages listing ordered by name
            </a>
    """
    if instanceid is None:
        request = context['request']
        instanceid = request.cradmin_instance.id
    return reverse_cradmin_url(
        instanceid=instanceid,
        appname=appname,
        roleid=roleid,
        viewname=viewname,
        args=args, kwargs=kwargs)


@register.filter
def cradmin_jsonencode(json_serializable_pythonobject):
    """
    Template filter that converts a json serializable object
    to a json encoded string.
    """
    return json.dumps(json_serializable_pythonobject)


@register.simple_tag(takes_context=True)
def cradmin_theme_staticpath(context):
    """
    """
    if 'request' in context:
        request = context['request']
        theme_path = None
        if hasattr(request, 'cradmin_instance'):
            theme_path = request.cradmin_instance.get_cradmin_theme_path()
            if theme_path:
                theme_path = str(theme_path)
        if not theme_path:
            theme_path = getattr(settings,
                                 'DJANGO_CRADMIN_THEME_PATH',
                                 'django_cradmin/{version}/styles/cradmin_theme_full/main.css'.format(
                                     version=django_cradmin.__version__
                                 ))
        return static(theme_path)
    else:
        raise Exception('The cradmin_theme_staticpath requires "request" to be in the template '
                        'context. You can get this using the "django.template.context_processors.request" '
                        'context processor.')


@register.simple_tag(takes_context=True)
def cradmin_render_renderable(context, renderable, include_context=False, **kwargs):
    """
    Render a :class:`django_cradmin.renderable.AbstractRenderable`.

    Unlike just using ``{{ renderable.render }}``, this
    sends the ``request`` into render (so this is the same
    as calling ``renderable.render(request=context['request'])``.

    Examples:

        Render a renderable named ``renderable`` in the current template context:

        .. code-block:: htmldjango

            {% load cradmin_tags %}

            {% cradmin_render_renderable renderable %}
    """
    request = context.get('request', None)
    full_kwargs = {}
    if include_context:
        full_kwargs['extra_context_data'] = context.flatten()
    full_kwargs.update(kwargs)
    return renderable.render(request=request, **full_kwargs)


@register.simple_tag
def cradmin_test_css_class(suffix):
    """
    Adds a CSS class for automatic tests. CSS classes added using this
    template tag is only included when the the :setting:`DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES`
    setting is set to ``True``.

    To use this template tag, you provide a ``suffix`` as input,
    and the output will be `` test-<suffix> ``. Notice that we
    include space before and after the css class - this means that you do not need to
    add any extra spaces within your class-attribute to make room for the
    automatic test only css class.

    Examples:

        Use the template tag to add test only css classes:

        .. code-block:: django

            {% load cradmin_tags %}

            <p class="paragraph paragraph--large{% cradmin_test_css_class 'introduction' %}">
                The introduction
            </p>

        Ensure that your test settings have ``DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES = True``.

        Write tests based on the test css class::

            from django import test
            import htmls

            class TestCase(test.TestCase):

                def test_introduction(self):
                    response = some_code_to_get_response()
                    selector = htmls.S(response.content)
                    with self.assertEqual(
                        'The introduction',
                        selector.one('test-introduction')

    Args:
        suffix: The suffix for your css class. The actual css class will be `` test-<suffix> ``.
    """
    include_test_css_classes = crsettings.get_setting('DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES', False)
    if include_test_css_classes:
        return '  test-{}  '.format(suffix)
    else:
        return ''



from . import blocklist


class UnsupportedMessageLevel(Exception):
    """
    Raised by :class:`.AbstractMessageContainerMixin` when an unsupported
    message level is requested.
    """


class AbstractMessageContainerMixin(object):
    """
    Defines the interface for message lists.

    This is an abstract mixin. To use it, you inherit this mixin and
    :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`
    (or a subclass of it), and override :meth:`.create_message_container`.
    """
    #: Supported message levels for :meth:`~.AbstractMessageContainerMixin.add_message`.
    supported_message_levels = {'warning', 'success', 'info'}

    @property
    def should_render(self):
        return self.get_childcount() > 0

    def validate_message_level(self, level):
        """
        Validate that the provided ``level`` is in :obj:`.supported_message_levels`.

        Raises:
            .UnsupportedMessageLevel: If ``level`` is not in :obj:`.supported_message_levels`.
        """
        if level not in self.supported_message_levels:
            raise UnsupportedMessageLevel('{module}.{classname} does not support message level: {level}'.format(
                module=self.__class__.__module__,
                classname=self.__class__.__name__,
                level=level))

    def create_message_container(self, level, text='', **kwargs):
        """
        Create a message container object.

        The returned object must be a subclass of
        :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.

        Args:
            level: One of the message levels in :obj:`.supported_message_levels`.
            text: Message text. This is optional. The only use-case for not including
                the message text is to create a more complex message with
                child containers. In that case, you should add child containers
                via ``kwargs``.
            **kwargs: Kwargs for the message container. This means that you
                can create really complex messages by adding any
                AbstractContainerRenderable subclass as a child of the message container.

        Raise:
            .UnsupportedMessageLevel: If ``level`` is not in :obj:`.supported_message_levels` -
                you must implement this behavior in your subclass by calling
                :meth:`.validate_message_level`.

        Returns:
            The created AbstractContainerRenderable object.
        """
        raise NotImplementedError()

    def add_message(self, **kwargs):
        """
        Add a message.

        Creates the message using :meth:`.create_message_container`,
        and adds the message as a child of this container.

        Args:
            **kwargs:: Kwargs for :meth:`.create_message_container`.
        """
        message_container = self.create_message_container(**kwargs)
        return self.add_child(message_container)

    def add_warning(self, **kwargs):
        """
        Add warning message.

        Args:
            **kwargs: Same as for :meth:`.add_message`, the only
                difference is that the ``level`` kwarg is set.
        """
        kwargs['level'] = 'warning'
        return self.add_message(**kwargs)

    def add_success(self, **kwargs):
        """
        Add success message.

        Args:
            **kwargs: Same as for :meth:`.add_message`, the only
                difference is that the ``level`` kwarg is set.
        """
        kwargs['level'] = 'success'
        return self.add_message(**kwargs)

    def add_info(self, **kwargs):
        """
        Add info message.

        Args:
            **kwargs: Same as for :meth:`.add_message`, the only
                difference is that the ``level`` kwarg is set.
        """
        kwargs['level'] = 'info'
        return self.add_message(**kwargs)

    def add_validationerror(self, validationerror, prefix=None, **kwargs):
        """
        Add a :class:`django.forms.ValidationError`.

        Args:
            validationerror: A :class:`django.forms.ValidationError`
            **kwargs: Same as for :meth:`.add_warning`.
        """
        for message in validationerror.messages:
            if prefix:
                message = '{}: {}'.format(prefix, message)
            self.add_warning(text=message, **kwargs)
        return self

    def add_validationerror_list(self, validationerror_list, **kwargs):
        """
        Add a list of :class:`django.forms.ValidationError` objects.

        Just a shortcut for looping through the list and adding errors
        using :meth:`.add_validationerror`.

        Args:
            validationerror_list: A list of :class:`django.forms.ValidationError` objects.
            **kwargs: Same as for :meth:`.add_validationerror`.
        """
        for validationerror in validationerror_list:
            self.add_validationerror(validationerror=validationerror, **kwargs)
        return self


class MessageContainer(blocklist.BlocklistItem):
    """
    Single message container.

    This is a very thin extension of :class:`django_cradmin.uicontainer.blocklist.BlocklistItem`
    that just adds unit test css classes.
    """
    def __init__(self, level, **kwargs):
        """
        Args:
            level: The message level. See :meth:`.AbstractMessageContainerMixin.create_message_container`.
            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.level = level
        super(MessageContainer, self).__init__(**kwargs)

    def get_default_bem_variant_list(self):
        """
        Default BEM variant set to the level kwarg for :meth:`.__init__`.

        So this assumes that a variant matching the level exists.
        """
        return [self.level]

    def get_default_test_css_class_suffixes_list(self):
        return super(MessageContainer, self).get_default_test_css_class_suffixes_list() + [
            '{}-message'.format(self.level or 'default')
        ]


class MessagesContainer(AbstractMessageContainerMixin, blocklist.Blocklist):
    """
    Messages container - contains zero or more :class:`.MessageContainer`.

    This is a very thin extension of :class:`django_cradmin.uicontainer.blocklist.Blocklist`
    that:

    - Changes the default variant to :obj:`~django_cradmin.uicontainer.blocklist.Blocklist.VARIANT_TIGHT`.
    - Implements :class:`.AbstractMessageContainerMixin` with
      :class:`.MessageContainer` as the message container.
    """
    def create_message_container(self, level, text='', **kwargs):
        return MessageContainer(level=level, text=text, **kwargs)

    def get_default_bem_variant_list(self):
        return ['tight']



from django import test

from django_cradmin.utils import cradmin_collections


class TestOrderedSet(test.TestCase):
    def test_empty_sanity(self):
        ordered_set = cradmin_collections.OrderedSet()
        self.assertEqual([], list(ordered_set))

    def test_sanity(self):
        ordered_set = cradmin_collections.OrderedSet()
        ordered_set.add('a')
        ordered_set.add('b')
        self.assertEqual(['a', 'b'], list(ordered_set))

    def test_duplicates(self):
        ordered_set = cradmin_collections.OrderedSet()
        ordered_set.add('a')
        ordered_set.add('b')
        ordered_set.add('a')
        self.assertEqual(['a', 'b'], list(ordered_set))



from __future__ import unicode_literals
import json
from xml.sax.saxutils import quoteattr
from django import forms
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.utils.translation import gettext
from django_cradmin.utils import crhumanize


class BulkFileUploadWidget(forms.Widget):
    template_name = 'django_cradmin/apps/cradmin_temporaryfileuploadstore/bulkfileupload-widget.django.html'

    def __init__(self,
                 accept=None,
                 apiparameters=None,
                 dropbox_text=None,
                 invalid_filetype_message=None,
                 advanced_fileselectbutton_text=None,
                 simple_fileselectbutton_text=None,
                 autosubmit=False,
                 overlaymode_autosubmit_uploading_message=None):
        """
        Parameters:
            accept (str): Comma separated string of filetypes that we should accept.
                Added to the file upload field as the accept attribute.
                Note that this is not validated serverside - use the accept attribute
                of ``apiparameters`` to do that.
            apiparameters (dict): Dict of API parameters. Here you can
                put values for the ``minutes_to_live``, ``accept``, ``max_filename_length``
                and ``unique_filenames`` attributes of
                :class:`django_cradmin.apps.cradmin_temporaryfileuploadstore.models.TemporaryFileCollection`.

                Example::

                    apiparameters = {
                        'minutes_to_live': 10,
                        'accept': 'application/pdf,text/plain,image/*',
                        'unique_filenames': True
                    }
            autosubmit: If this is ``True``, the form is submitted
                when the upload is finished. This also makes it impossible
                to upload more than one batch of files, because the widget
                hides all the upload widgets as soon as any files is added.
                It works with multi-file upload, but the user will only
                be able to upload one batch of files, and they will not
                be able to remove or upload more files.

            overlaymode_autosubmit_uploading_message
                The message to show submitting the form when using overlay mode and autosubmit.

                This is just to ensure that the user understands that the
                request is beeing submitted, and that they should wait and not
                close the browser window.

                Only used if ``autosubmit`` is ``True``, and the form uses the
                form uses ``django-cradmin-bulkfileupload-form-overlay="true"``.

        """
        self.accept = accept
        self.apiparameters = apiparameters or {}
        self.dropbox_text = dropbox_text
        self.invalid_filetype_message = invalid_filetype_message
        self.advanced_fileselectbutton_text = advanced_fileselectbutton_text
        self.simple_fileselectbutton_text = simple_fileselectbutton_text
        self.autosubmit = autosubmit
        self.overlaymode_autosubmit_uploading_message = overlaymode_autosubmit_uploading_message

        super(BulkFileUploadWidget, self).__init__(attrs=None)

    def get_max_filesize_bytes(self):
        max_filesize_bytes = self.apiparameters.get('max_filesize_bytes', None)
        return max_filesize_bytes

    def get_max_filesize_bytes_exceeded_errormessage(self):
        max_filesize_bytes = self.get_max_filesize_bytes()
        if max_filesize_bytes is not None:
            return gettext('Can not upload files larger than %(max_filesize)s.') % {
                'max_filesize': crhumanize.human_readable_filesize(max_filesize_bytes)
            }

    def get_uploadapiurl(self):
        """
        Get the file upload API URL.

        You can override this if you provide your own upload API.
        """
        return reverse('cradmin_temporary_file_upload_api')

    def get_errormessage503(self):
        """
        Get the error message to show on 503 server errors.
        """
        return gettext('Server timeout while uploading the file. This may be caused '
                       'by a poor upload link and/or a too large file.')

    def get_use_singlemode(self):
        """
        If this returns ``True``, only single file upload is allowed.
        Defaults to ``False``, but :class:`.SingleFileUploadWidget`
        overrides this.
        """
        return False

    def get_apiparameters(self):
        """
        Get parameters for the upload API.
        """
        apiparameters = self.apiparameters.copy()
        apiparameters['singlemode'] = self.get_use_singlemode()
        return apiparameters

    def get_angularjs_directive_options(self):
        """
        Get options for the ``django-cradmin-bulkfileupload``
        angularjs directive.

        Must return a JSON encodable dict.
        """
        return {
            'uploadapiurl': self.get_uploadapiurl(),
            'apiparameters': self.get_apiparameters(),
            'errormessage503': self.get_errormessage503(),
            'autosubmit': self.autosubmit,
            'close_errormessage_label': gettext('Close'),
            'remove_file_label': gettext('Remove'),
            'removing_file_message': gettext('Removing ...'),
        }

    def __get_rejected_files_errormessage_map(self):
        return {
            'invalid_filetype': str(self.invalid_filetype_message),
            'max_filesize_bytes_exceeded': self.get_max_filesize_bytes_exceeded_errormessage(),
        }

    def get_template_context_data(self, **context):
        """
        Can be overridden to adjust the template context data.
        """
        context['accept'] = self.accept
        context['dropbox_text'] = self.dropbox_text
        context['rejected_files_errormessage_map'] = quoteattr(json.dumps(
            self.__get_rejected_files_errormessage_map()))
        context['advanced_fileselectbutton_text'] = self.advanced_fileselectbutton_text
        context['simple_fileselectbutton_text'] = self.simple_fileselectbutton_text
        context['angularjs_directive_options'] = quoteattr(json.dumps(
            self.get_angularjs_directive_options()))
        context['singlemode'] = self.get_use_singlemode()
        context['overlaymode_autosubmit_uploading_message'] = self.overlaymode_autosubmit_uploading_message
        return context

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        return render_to_string(self.template_name, self.get_template_context_data(
            hiddenfieldname=name,
            fieldvalue=value))


class SingleFileUploadWidget(BulkFileUploadWidget):
    def __init__(self, *args, **kwargs):
        super(SingleFileUploadWidget, self).__init__(*args, **kwargs)

    def get_use_singlemode(self):
        return True

    def get_template_context_data(self, **context):
        context = super(SingleFileUploadWidget, self).get_template_context_data(**context)
        return context



from . import form_mixins
from . import container


class AbstractLabel(container.AbstractContainerRenderable):
    template_name = 'django_cradmin/uicontainer/label/label.django.html'

    def get_default_html_tag(self):
        return 'label'

    @property
    def for_attribute(self):
        return False

    def get_html_element_attributes(self):
        html_element_attributes = super(AbstractLabel, self).get_html_element_attributes()
        html_element_attributes['for'] = self.for_attribute
        return html_element_attributes

    @property
    def label_text(self):
        raise NotImplementedError()

    @property
    def field_renderable(self):
        raise NotImplementedError()

    def should_show_text_after_field(self):
        return False


class Label(AbstractLabel, form_mixins.FieldWrapperRenderableChildMixin):
    """
    Renders a label for a :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`.

    You never use this on its own outside a
    :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`.
    """
    def should_include_for_attribute(self):
        return self.field_wrapper_renderable.field_renderable.should_have_for_attribute_on_label()

    def get_default_dom_id(self):
        return '{field_dom_id}_label'.format(
            field_dom_id=self.field_wrapper_renderable.field_renderable.dom_id
        )

    def get_default_bem_block_or_element(self):
        """
        Default BEM block is ``label``.
        """
        return 'label'

    @property
    def for_attribute(self):
        if self.should_include_for_attribute():
            if self.field_wrapper_renderable.field_renderable.should_render_as_child_of_label():
                return False
            else:
                return self.field_wrapper_renderable.field_renderable.dom_id
        else:
            return False

    @property
    def label_text(self):
        bound_formfield = self.field_wrapper_renderable.bound_formfield
        if bound_formfield.field.label:
            return bound_formfield.field.label
        else:
            return bound_formfield.name

    @property
    def field_renderable(self):
        if self.field_wrapper_renderable.field_renderable.should_render_as_child_of_label():
            return self.field_wrapper_renderable.field_renderable
        else:
            return None


class SubWidgetLabel(AbstractLabel, form_mixins.FieldChildMixin):
    """
    Basic label for a Django SubWidget.

    Assumes that we want ot render the Django SubWidget within the label.

    If you want
    """
    def __init__(self, subwidget_field_renderable, **kwargs):
        self.subwidget_field_renderable = subwidget_field_renderable
        super(SubWidgetLabel, self).__init__(**kwargs)

    def prepopulate_virtual_children_list(self):
        return [
            self.subwidget_field_renderable
        ]

    def should_show_text_after_field(self):
        return True

    def get_default_dom_id(self):
        return '{field_dom_id}_{index_in_parent}_label'.format(
            field_dom_id=self.field_renderable.dom_id,
            index_in_parent=self.subwidget_field_renderable.index_in_parent)

    @property
    def label_text(self):
        return self.subwidget_field_renderable.label_text

    @property
    def field_renderable(self):
        return self.subwidget_field_renderable


class StyledSubWidgetLabel(SubWidgetLabel):
    """
    A ``<label>`` for checkbox or radio list where we
    assumes that you want to hide the actual input field
    and style it with a _control indicator_ span.

    The control indicator span element css class will
    be ``<BEM block>__control-indicator`` - can be overriden
    in :meth:`~.StyledSubWidgetLabel.control_indicator_bem_element`.
    """
    template_name = 'django_cradmin/uicontainer/label/styled_sub_widget_label.django.html'

    @property
    def control_indicator_bem_element(self):
        return '{}__control-indicator'.format(self.get_bem_block_or_element())


class RadioSubWidgetLabel(StyledSubWidgetLabel):
    """
    A ``<label>`` suitable for wrapping around lists of
    radio buttons.
    """
    def get_default_bem_block_or_element(self):
        """
        Uses the ``radio`` BEM block by default.
        """
        return 'radio'

    def get_default_bem_variant_list(self):
        """
        Uses the ``radio--block`` BEM variant by default.
        """
        return ['block']


class CheckboxSubWidgetLabel(StyledSubWidgetLabel):
    """
    A ``<label>`` suitable for wrapping around lists of
    checkboxes.
    """
    def get_default_bem_block_or_element(self):
        """
        Uses the ``checkbox`` BEM block by default.
        """
        return 'checkbox'

    def get_default_bem_variant_list(self):
        """
        Uses the ``checkbox--block`` BEM variant by default.
        """
        return ['block']



from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib.auth.decorators import login_required

from django_cradmin.apps.cradmin_temporaryfileuploadstore.views.temporary_file_upload_api import \
    UploadTemporaryFilesView

urlpatterns = [
    url(r'^temporary_file_upload_api$',
        login_required(UploadTemporaryFilesView.as_view()),
        name='cradmin_temporary_file_upload_api'),
]



from django import test

from django_cradmin import javascriptregistry


class JsComponent1(javascriptregistry.component.AbstractJsComponent):
    @classmethod
    def get_component_id(cls):
        return 'jscomponent1'


class JsComponent2(javascriptregistry.component.AbstractJsComponent):
    @classmethod
    def get_component_id(cls):
        return 'jscomponent2'


class TestRegistry(test.TestCase):

    def test_add_duplicate_component_id(self):
        class AnotherJsComponent1(javascriptregistry.component.AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'jscomponent1'

        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        with self.assertRaises(javascriptregistry.DuplicateComponentId):
            registry.add(AnotherJsComponent1)

    def test_add_ok(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertIn('jscomponent1', registry._jscomponent_classes)

    def test_contains_class_false(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertNotIn(JsComponent2, registry)

    def test_contains_class_true(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertIn(JsComponent1, registry)

    def test_contains_str_false(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertNotIn(JsComponent2.get_component_id(), registry)

    def test_contains_str_true(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertIn(JsComponent1.get_component_id(), registry)

    def test_getitem_failed(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertIn(JsComponent1.get_component_id(), registry)

    def test_getitem_ok(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        self.assertEqual(registry[JsComponent1.get_component_id()], JsComponent1)

    def test_get_component_objects_invalid_component_id(self):
        registry = javascriptregistry.MockableRegistry()
        with self.assertRaises(KeyError):
            registry.get_component_objects(
                request=None,
                component_ids=[JsComponent1.get_component_id()])

    def test_get_component_objects_simple(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        component_objects = registry.get_component_objects(
            request=None,
            component_ids=[JsComponent1.get_component_id()])
        component_ids = [component.component_id for component in component_objects]
        self.assertEqual([JsComponent1.get_component_id()], component_ids)

    def test_get_component_objects_duplicates(self):
        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        component_objects = registry.get_component_objects(
            request=None,
            component_ids=[JsComponent1.get_component_id(), JsComponent1.get_component_id()])
        component_ids = [component.component_id for component in component_objects]
        self.assertEqual([JsComponent1.get_component_id()], component_ids)

    def test_get_component_objects_with_dependencies(self):
        class MockJsComponent(javascriptregistry.component.AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'mock'

            def get_dependencies(self):
                return [JsComponent1.get_component_id(), JsComponent2.get_component_id()]

        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        registry.add(JsComponent2)
        registry.add(MockJsComponent)
        component_objects = registry.get_component_objects(
            request=None,
            component_ids=[MockJsComponent.get_component_id()])
        component_ids = [component.component_id for component in component_objects]
        self.assertEqual(
            [
                JsComponent1.get_component_id(),
                JsComponent2.get_component_id(),
                MockJsComponent.get_component_id(),
            ],
            component_ids)

    def test_get_component_objects_with_dependency_duplicates(self):
        class MockJsComponent1(javascriptregistry.component.AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'mock1'

            def get_dependencies(self):
                return [JsComponent1.get_component_id(), JsComponent2.get_component_id()]

        class MockJsComponent2(javascriptregistry.component.AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'mock2'

            def get_dependencies(self):
                return [JsComponent2.get_component_id()]

        registry = javascriptregistry.MockableRegistry()
        registry.add(JsComponent1)
        registry.add(JsComponent2)
        registry.add(MockJsComponent1)
        registry.add(MockJsComponent2)
        component_objects = registry.get_component_objects(
            request=None,
            component_ids=[MockJsComponent1.get_component_id(), MockJsComponent2.get_component_id()])
        component_ids = [component.component_id for component in component_objects]
        self.assertEqual(
            [
                JsComponent1.get_component_id(),
                JsComponent2.get_component_id(),
                MockJsComponent1.get_component_id(),
                MockJsComponent2.get_component_id(),
            ],
            component_ids)



from django import template
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.safestring import mark_safe

register = template.Library()


def _get_kss_sections(kss_styleguide, prefix=None):
    sections = kss_styleguide.sections.values()
    if prefix:
        sections = filter(lambda s: s.reference.startswith(prefix), sections)
    sections = sorted(sections, key=lambda s: s.reference)
    return sections


def _kss_section_level(section):
    return section.reference.count('.') + 1


class KssSectionTree(object):
    def __init__(self, sectiontree):
        self._sectiontree = []
        self._previous_node = sectiontree
        for node in sectiontree.sorted_all_descendants_flat():
            self.add(node)
            self._previous_node = node
        if self._previous_node != sectiontree:
            for level in range(self._previous_node.level - 1):
                self._sectiontree.append('leveldown')

    def add(self, node):
        if self._previous_node.level != node.level and self._previous_node.level != -1:
            if node.level > self._previous_node.level:
                self._sectiontree.append('levelup')
            else:
                self._sectiontree.append('leveldown')
        self._sectiontree.append(node)

    def __iter__(self):
        return iter(self._sectiontree)


@register.filter()
def kss_section_level(section):
    return _kss_section_level(section)


@register.simple_tag()
def kss_section_url(styleguideconfig, node):
    levels = 2
    if node.level == 0:
        levels = 1
    prefix = '.'.join(node.reference.split('.')[0:levels])
    url = reverse('cradmin_kss_styleguide_guide', kwargs={
        'unique_id': styleguideconfig.unique_id,
        'prefix': prefix
    })
    return '{}#kssref-{}'.format(url, node.reference)


@register.simple_tag()
def render_kss_section_description(styleguideconfig, section):
    description = styleguideconfig.format_description(section)
    return mark_safe(description)


@register.simple_tag()
def render_kss_section_example(styleguideconfig, section):
    return mark_safe(styleguideconfig.format_example(section))


@register.simple_tag(takes_context=True)
def render_kss_section(
        context, styleguideconfig, node):
    return render_to_string(
        template_name=styleguideconfig.get_section_template_name(),
        context={
            'styleguideconfig': styleguideconfig,
            'node': node,
        },
        request=context.get('request', None)
    )


@register.simple_tag(takes_context=True)
def render_kss_sections(
        context, styleguideconfig, kss_styleguide, prefix=None):
    node = kss_styleguide.as_tree()
    if prefix:
        node = node.get_node_by_reference(prefix)

    return render_to_string(
        template_name=styleguideconfig.get_sections_template_name(),
        context={
            'styleguideconfig': styleguideconfig,
            'node': node,
        },
        request=context.get('request', None)
    )


@register.simple_tag(takes_context=True)
def render_kss_toc_node(context, styleguideconfig, node):
    return render_to_string(
        template_name=styleguideconfig.get_toc_node_template_name(),
        context={
            'styleguideconfig': styleguideconfig,
            'node': node,
        },
        request=context.get('request', None)
    )


@register.simple_tag(takes_context=True)
def render_kss_toc(context, styleguideconfig, kss_styleguide):
    sectiontree = kss_styleguide.as_tree()
    return render_to_string(
        template_name=styleguideconfig.get_toc_template_name(),
        context={
            'styleguideconfig': styleguideconfig,
            'sectiontree': sectiontree,
        },
        request=context.get('request', None)
    )



from django_cradmin import crapp


class FormViewMixin:
    """
    Mixin class for form views.
    """

    #: Get the view name for the listing page.
    #: You can set this, or implement :meth:`.get_listing_url`.
    #: Defaults to :obj:`django_cradmin.crapp.INDEXVIEW_NAME`.
    listing_viewname = crapp.INDEXVIEW_NAME

    def get_pagetitle(self):
        """
        Get the page title (the title tag).
        """
        return ''

    def get_pageheading(self):
        """
        Get the page heading.

        Defaults to :meth:`.get_pagetitle`.
        """
        return self.get_pagetitle()

    def get_form_renderable(self):
        """
        Get a :class:`django_cradmin.renderable.AbstractRenderable` that renders
        the form.

        This will typically be a :doc:`uicontainer` tree containing a
        :class:`django_cradmin.uicontainer.form.Form`, but it can be any
        AbstractRenderable. Not using a :class:`django_cradmin.uicontainer.form.Form`
        (or a subclass of it) is fairly complex when it comes to handling error
        messages and form rendering, so it is generally not recommended.

        See :meth:`django_cradmin.viewhelpers.formview.formview.FormView` for examples.

        Returns:
            django_cradmin.renderable.AbstractRenderable: The renderable object.
        """
        raise NotImplementedError()

    def add_formview_mixin_context_data(self, context):
        """
        Must be used by get_context_data() in subclasses to add the context data required to
        render the view.

        Examples:

            Adding the required context data::

                def get_context_data(self, **kwargs):
                    context = super(MyView, self).get_context_data(**kwargs)
                    self.add_formview_mixin_context_data(context=context)
                    return context
        """
        context['form_renderable'] = self.get_form_renderable()
        context['pagetitle'] = self.get_pagetitle()
        context['pageheading'] = self.get_pageheading()

    def get_listing_url(self):
        """
        Get the URL of the listing view.

        Defaults to :obj:`.listing_viewname`. This is used as the success URL
        by default.
        """
        return self.request.cradmin_app.reverse_appurl(self.listing_viewname)

    def get_default_save_success_url(self):
        """
        Get the save success URL.

        Defaults to :meth:`.get_listing_url`, but if ``success_url`` is in
        ``request.GET``, that is used instead.
        """
        if 'success_url' in self.request.GET:
            return self.request.GET['success_url']
        else:
            return self.get_listing_url()

    def get_success_url(self):
        """
        Defaults to :meth:`.get_default_save_success_url`.
        """
        return self.get_default_save_success_url()



from django.core.management.base import BaseCommand
from django_cradmin.apps.cradmin_email import emailutils


class Command(BaseCommand):
    def handle(self, args, **kwargs):
        class DemoEmail(emailutils.AbstractEmail):
            subject_template = 'cradmin_email/cradmin_email_send_testmail/subject.django.txt'
            html_message_template = 'cradmin_email/cradmin_email_send_testmail/html_message.django.html'

        DemoEmail(recipient='post@appresso.no',
                  extra_context_data={'name': 'Test Name'}).send()



from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import UpdateView as DjangoUpdateView
from django_cradmin import javascriptregistry
from django_cradmin.viewhelpers.mixins import QuerysetForRoleMixin

from . import create_update_view_mixin


class UpdateViewMixin(create_update_view_mixin.CreateUpdateViewMixin):
    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Defaults to ``Edit <verbose_name model>``.
        """
        return _('Edit %(what)s') % {'what': self.model_verbose_name}

    def get_success_message(self, obj):
        return _('Saved "%(object)s"') % {'object': obj}


class WithinRoleUpdateView(QuerysetForRoleMixin,
                           UpdateViewMixin,
                           DjangoUpdateView,
                           javascriptregistry.viewmixin.WithinRoleViewMixin):
    template_name = 'django_cradmin/viewhelpers/formview/within_role_update_view.django.html'

    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Defaults to ``Edit <verbose_name model>``.
        """
        return _('Edit %(what)s') % {'what': self.model_verbose_name}

    def get_success_message(self, obj):
        return _('Saved "%(object)s"') % {'object': obj}

    def get_context_data(self, **kwargs):
        context = super(WithinRoleUpdateView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        self.add_create_update_view_mixin_context_data(context=context)
        return context


class UpdateRoleView(WithinRoleUpdateView):
    """
    Extends :class:`.UpdateView` to streamline editing the current role
    object.

    Just like :class:`.UpdateView`, but with the get_object and
    get_queryset_for_role methods implemented to edit the current role
    object.
    """
    def get_object(self, queryset=None):
        return self.get_queryset_for_role().get()

    def get_queryset_for_role(self):
        return self.get_model_class().objects.filter(pk=self.request.cradmin_role.pk)


class RedirectToCreateIfDoesNotExistMixin(object):
    """
    An update view mixin that redirects to a create view when the
    object requested does not exist.

    You will typically use this for objects with a OneToOne relationship
    to the current role, but that may not exist. Then you would use
    something like::

        class MyUpdateView(update.UpdateView, update.RedirectToCreateIfDoesNotExistMixin):
            def get_object(self, queryset=None):
                return self.get_queryset_for_role().get()

            def get_queryset_for_role(self):
                return self.get_model_class().objects.filter(
                    someonetooneattr=self.request.cradmin_role)

    And the view will automatically redirect to the create view
    if the object does not exist.
    """

    #: The viewname within this app for the create view.
    #: See :meth:`.get_createurl`. Defaults to ``create``.
    createview_appurl_name = 'create'

    def get_createurl(self):
        """
        Get the URL of the create view that you want to redirect to if
        the requested object does not exist.

        Defaults to::

            self.request.cradmin_app.reverse_appurl(self.createview_appurl_name)
        """
        return self.request.cradmin_app.reverse_appurl(self.createview_appurl_name)

    def get(self, request, *args, **kwargs):
        try:
            return super().get(request, *args, **kwargs)
        except self.get_model_class().DoesNotExist:
            return HttpResponseRedirect(self.request.cradmin_app.reverse_appurl('create'))



from __future__ import unicode_literals

from django_cradmin.python2_compatibility import mock

from django.test import TestCase
from future import standard_library
import htmls

from django_cradmin.viewhelpers.listfilter.base.abstractfilterlist import AbstractFilterList
from django_cradmin.viewhelpers.listfilter.basefilters.single.abstractselect import AbstractSelectFilter

standard_library.install_aliases()


class TestAbstractSelectFilter(TestCase):
    def test_render_sanity(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('', 'Do not apply'),
                    ('first', 'First choice'),
                    ('second', 'Second choice'),
                ]

        testfilter = MySelectFilter(slug='test')
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertTrue(selector.exists('#django_cradmin_listfilter_test.django-cradmin-listfilter-filter'))
        self.assertTrue(selector.exists('select#django_cradmin_listfilter_test_input'))
        self.assertEqual(3, selector.count('option'))

    def test_render_option_value_empty(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('', 'Do not apply'),
                ]

        testfilter = MySelectFilter(slug='test')
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: '/test/{}'.format(filters_string),
            target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual('/test/', selector.one('option')['value'])

    def test_render_option_value_nonempty(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('somevalue', 'Test'),
                ]

        testfilter = MySelectFilter(slug='test')
        filterlist = AbstractFilterList(
            urlbuilder=lambda filters_string: '/test/{}'.format(filters_string),
            target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual('/test/test-somevalue', selector.one('option')['value'])

    def test_render_option_label(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('somevalue', 'Test'),
                ]

        testfilter = MySelectFilter(slug='test')
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual('Test', selector.one('option').alltext_normalized)

    def test_render_option_selected(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('', 'Do not apply'),
                    ('first', 'First choice'),
                    ('second', 'Second choice'),
                ]

        testfilter = MySelectFilter(slug='test')
        testfilter.set_values(values=['first'])
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual(
            'First choice',
            selector.one('option[selected]').alltext_normalized)

    def test_render_option_no_value_select_first(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('', 'Do not apply'),
                    ('first', 'First choice'),
                    ('second', 'Second choice'),
                ]

        testfilter = MySelectFilter(slug='test')
        testfilter.set_values(values=[])
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual(
            'Do not apply',
            selector.one('option[selected]').alltext_normalized)

    def test_render_label_no_label(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('unused', 'unused'),
                ]

        testfilter = MySelectFilter(slug='test')
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertFalse(selector.exists('label'))

    def test_render_label_has_label(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('unused', 'unused'),
                ]

        testfilter = MySelectFilter(slug='test', label='A label')
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertEqual('A label', selector.one('label').alltext_normalized)
        self.assertEqual('django_cradmin_listfilter_test_input', selector.one('label')['for'])

    def test_render_label_get_label_is_screenreader_only_true(self):
        class MySelectFilter(AbstractSelectFilter):
            def get_choices(self):
                return [
                    ('unused', 'unused'),
                ]

            def get_label_is_screenreader_only(self):
                return True

        testfilter = MySelectFilter(slug='test', label='A label')
        filterlist = AbstractFilterList(urlbuilder=mock.MagicMock(), target_dom_id='testdomid')
        filterlist.append(testfilter)
        selector = htmls.S(testfilter.render())
        self.assertTrue(selector.exists('label'))
        self.assertTrue(selector.one('label').hasclass('screenreader-only'))



from pkg_resources import resource_string
import json
import sys

default_app_config = 'django_cradmin.appconfig.CradminAppConfig'

if sys.version_info.major == 2:
    __version__ = json.loads(resource_string(__name__, 'version.json'))
else:
    __version__ = json.loads(resource_string(__name__, 'version.json').decode('utf-8'))



from . import container


class AbstractWithOptionalEscapedText(container.AbstractContainerRenderable):
    """
    A renderable that has an optional ``text`` kwarg
    that adds the provided text as a :class:`django_cradmin.uicontainer.text.EscapedText`
    child.
    """
    def __init__(self, text=None, **kwargs):
        """
        Args:
            text: Optional text to add to the container. See :meth:`.prepopulate_children_list`.
            **kwargs: Kwargs for :class:`~django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.text = text
        super(AbstractWithOptionalEscapedText, self).__init__(**kwargs)

    def prepopulate_children_list(self):
        """
        Automatically adds a :class:`django_cradmin.uicontainer.text.EscapedText`
        child if the ``text`` kwarg for :meth:`.__init__` is set to a value
        that is ``bool(text) == True``.
        """
        from . import text
        if self.text:
            return [
                text.EscapedText(text=self.text)
            ]
        else:
            return []


class AbstractWithOptionalParagraphWithEscapedText(container.AbstractContainerRenderable):
    """
    A renderable that has an optional ``text`` kwarg
    that adds the provided text as a :class:`django_cradmin.uicontainer.typography.Paragraph`
    child containing a :class:`django_cradmin.uicontainer.text.EscapedText` child
    """
    def __init__(self, text=None, **kwargs):
        """
        Args:
            text: Optional text to add to the container. See :meth:`.prepopulate_children_list`.
            **kwargs: Kwargs for :class:`~django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.text = text
        super(AbstractWithOptionalParagraphWithEscapedText, self).__init__(**kwargs)

    def prepopulate_children_list(self):
        """
        Automatically adds a :class:`django_cradmin.uicontainer.typography.Paragraph`
        child with a :class:`django_cradmin.uicontainer.text.EscapedText`
        child if the ``text`` kwarg for :meth:`.__init__` is set to a value
        that is ``bool(text) == True``.
        """
        from . import text
        from . import semantic
        if self.text:
            return [
                semantic.Paragraph(
                    children=[text.EscapedText(text=self.text)])
            ]
        else:
            return []



from .container import AbstractContainerRenderable
from . import form_mixins


class FieldSet(AbstractContainerRenderable, form_mixins.FormRenderableChildMixin):
    template_name = 'django_cradmin/uicontainer/fieldset.django.html'

    def __init__(self, title, **kwargs):
        self.title = title
        super(FieldSet, self).__init__(**kwargs)

    def get_default_html_tag(self):
        return 'fieldset'



from __future__ import unicode_literals

from django.utils.translation import pgettext_lazy

from django_cradmin import renderable
from django_cradmin.viewhelpers import listbuilder


class LinkItemRenderable(renderable.AbstractRenderableWithCss):
    """
    Use this to add links to the menu.
    """
    template_name = 'django_cradmin/crmenu/menuitem/link.django.html'

    def __init__(self, label, url, is_active=False):
        """
        Parameters:
            label: A label shown in the menu.
            url: The url to go to whem the user clicks the menu item.
            is_active: Should be ``True`` if the menuitem should be styled as active.
        """
        self.label = label
        self.url = url
        self.is_active = is_active

    def get_base_css_classes_list(self):
        css_classes = ['adminui-page-header__navlink']
        if self.is_active:
            css_classes.append('adminui-page-header__navlink--active')
        return css_classes


class ButtonLinkItemRenderable(LinkItemRenderable):
    """
    Use this to add links styled as a button to the menu.
    """
    def get_base_css_classes_list(self):
        css_classes = super(ButtonLinkItemRenderable, self).get_base_css_classes_list()
        css_classes.append('adminui-page-header__navlink--button')
        return css_classes


class MenuToggleItemItemRenderable(renderable.AbstractRenderableWithCss):
    """
    Use this to add an expandable menu toggle to the menu.
    """
    template_name = 'django_cradmin/crmenu/menuitem/menutoggle.django.html'

    def get_label(self):
        return pgettext_lazy('cradmin default header menu toggle label', 'Menu')

    def get_base_css_classes_list(self):
        return ['adminui-page-header__navmenutoggle']


class AbstractMenuRenderable(listbuilder.base.List):
    """
    Base class for rendering a menu.

    To get a completely custom HTML menu, you simply set your own template
    (see :meth:`django_cradmin.renderable.AbstractRenderable.get_template_name`)
    and write your own HTML.
    """
    def __init__(self, cradmin_instance):
        self.cradmin_instance = cradmin_instance
        super(AbstractMenuRenderable, self).__init__()

    def get_wrapper_htmltag(self):
        """
        Get the HTML tag to wrap the menu in.

        Defaults to ``"div"``.
        """
        return 'div'

    def get_wrapper_htmltag_id(self):
        """
        Get the ID of the wrapper HTML element.
        """
        raise None


class DefaultMainMenuRenderable(AbstractMenuRenderable):
    """
    The default large screen (desktop) :class:`.Menu` renderable.
    """
    template_name = 'django_cradmin/crmenu/menu/default-main.django.html'
    menutoggle_renderable_class = MenuToggleItemItemRenderable

    def get_wrapper_htmltag_id(self):
        return 'id_django_cradmin_menu_main'

    def get_menutoggle_renderable(self):
        return self.menutoggle_renderable_class()

    def get_base_css_classes_list(self):
        return ['adminui-page-header__nav']


class DefaultExpandableMenuRenderable(AbstractMenuRenderable):
    """
    The default small screen (mobile) :class:`.Menu` renderable.
    """
    template_name = 'django_cradmin/crmenu/menu/default-expandable.django.html'

    def get_wrapper_htmltag_id(self):
        return 'id_django_cradmin_menu_expandable'

    def get_base_css_classes_list(self):
        return ['adminui-expandable-menu']



from __future__ import unicode_literals

from django.conf import settings
from django.forms.utils import flatatt
from django_cradmin import renderable


class NotBootsrappedError(Exception):
    """
    Raised when trying to use features of
    :class:`.AbstractContainerRenderable`
    that requires is to have been bootstrapped.
    """


class AlreadyBootsrappedError(Exception):
    """
    Raised when trying to :meth:`~.AbstractContainerRenderable.bootstrap`
    and already bootstrapped :class:`.AbstractContainerRenderable`.
    """


class NotAllowedToAddChildrenError(Exception):
    """
    Raised when trying to add children to a :class:`.AbstractContainerRenderable`
    where :meth:`~.AbstractContainerRenderable.html_tag_supports_children`
    returns ``False``.
    """


class UnsupportedHtmlTagError(ValueError):
    """
    Raised when providing an invalid ``html_tag`` kwarg to :class:`.AbstractContainerRenderable`.

    See :obj:`.AbstractContainerRenderable.supported_html_tags`.
    """


class InvalidBemError(ValueError):
    """
    Raised when invalid BEM is supplied.
    """


class InvalidDomIdError(ValueError):
    """
    Raised when invalid dom_id is supplied.
    """


class AbstractContainerRenderable(renderable.AbstractRenderableWithCss):
    """
    Base class for all renderables in the uicontainer framework.

    This can not be used directly. You extend it, and at least override
    :meth:`.get_default_html_tag`, or use one of the subclasses.

    The most basic subclass is :class:`django_cradmin.uicontainer.div.Div`.

    .. attribute:: parent

        The parent AbstractContainerRenderable. Set in :meth:`.bootstrap`.
        The attribute does not exist if :meth:`.bootstrap` has not been
        run. Is ``None`` if this is the root of the container tree.

    .. attribute:: properties

        A dict of properties. These properties is copied down to the
        ``properties`` attribute of children (with the update-method, not full replace)
        in :meth:`.bootstrap`.
        This means that you can add properties in ``__init__()``,
        and make them available to any children recursively.
    """
    template_name = 'django_cradmin/uicontainer/container.django.html'

    #: You can override this to specify a set of supported HTML tags
    #: for the ``html_tag`` attribute for :meth:`~.AbstractContainerRenderable.__init__`.
    #: This is useful to avoid typing errors. It should not be a big problem if you
    #: forget a tag that should be supported - developers can just create a subclass.
    #:
    #: If the value of this field is None, or any other value that is considered False by
    #: ``bool()``, we do not validate the ``html_tag`` kwarg.
    supported_html_tags = None

    def __init__(self, children=None,
                 bem_block=None, bem_element=None, bem_variant_list=None,
                 html_tag=None,
                 css_classes_list=None,
                 extra_css_classes_list=None,
                 test_css_class_suffixes_list=None,
                 role=False,
                 dom_id=False,
                 html_element_attributes=None,
                 **kwargs):
        """
        Args:
            children: List of children. Children must be objects of subclasses
                of :class:`.AbstractContainerRenderable`.
            css_classes_list (list): Override the :meth:`default css classes <.get_default_css_classes_list>`
                with your own list of css classes.
            extra_css_classes_list (list): Add extra css classes. This is appended to
                the css classes in the ``css_classes_list`` kwarg if that is specified,
                or appended to the css classes returned by
                :meth:`.get_default_css_classes_list`.
            role (str): The value of the role attribute.
                If this is not specified, we fall back on the value returned
                by :meth:`.get_default_role`.
                If both is ``False``, we do not render the role attribute.
            dom_id (str): The value of the id attribute.
                If this is not specified, we fall back on the value returned
                by :meth:`.get_default_dom_id`.
                If both is ``False``, we do not render the id attribute.
            html_element_attributes (dict): HTML element attributes to add
                to the HTML element. This adds attributes returned
                by :meth:`.get_html_element_attributes`. If this dict includes
                attributes returned by :meth:`.get_html_element_attributes`,
                the attributes specified in this kwarg takes presedense.
                The format of the dict is specified in :meth:`.get_html_element_attributes`.
        """
        self.kwargs = kwargs
        self.validate_dom_id(dom_id=dom_id)
        self.validate_bem(bem_block=bem_block,
                          bem_element=bem_element)
        self.validate_html_tag(html_tag=html_tag)
        self._childrenlist = []
        self._virtual_childrenlist = []
        self._is_bootstrapped = False
        self.properties = {}
        self._overridden_bem_block_or_element = bem_block or bem_element
        self._overridden_bem_variant_list = bem_variant_list
        self._overridden_role = role
        self._overridden_dom_id = dom_id
        self._overridden_html_tag = html_tag
        self._html_element_attributes = html_element_attributes
        self._overridden_css_classes_list = css_classes_list
        self._overridden_test_css_class_suffixes_list = test_css_class_suffixes_list
        self._extra_css_classes_list = extra_css_classes_list
        self.add_children(*self.prepopulate_children_list())
        self.add_virtual_children(*self.prepopulate_virtual_children_list())
        if children:
            self.add_children(*children)

    def should_validate_dom_id(self):
        """
        Should we raise :class:`.InvalidDomIdError` exception
        when the ``dom_id`` kwarg is malformed.

        Returns the value of the :setting:`DJANGO_CRADMIN_UICONTAINER_VALIDATE_DOM_ID`
        setting, falling back to ``True`` if it is not defined.

        The validator requires the dom_id
        to start with ``id_``, be lowercase, and not contain ``-``.

        We recommend to not override this to ensure uniform DOM id naming.

        You should disable this validation in production using the
        :setting:`DJANGO_CRADMIN_UICONTAINER_VALIDATE_DOM_ID` setting.
        """
        return getattr(settings, 'DJANGO_CRADMIN_UICONTAINER_VALIDATE_DOM_ID', True)

    def should_validate_bem(self):
        """
        Should we raise :class:`.InvalidBemIdError` exception
        when the ``bem_block`` or ``bem_element`` kwarg is malformed?

        Returns the value of the :setting:`DJANGO_CRADMIN_UICONTAINER_VALIDATE_BEM`
        setting, falling back to ``True`` if it is not defined.

        The validator requires the bem_block to not contain ``__``
        (double underscore), and the bem_element to comtain ``__`` (double
        underscore).

        We recommend to not chanding this to ensure BEM elements and
        blocks are used correctly.

        You should disable this validation in production using the
        :setting:`DJANGO_CRADMIN_UICONTAINER_VALIDATE_BEM` setting.
        """
        return getattr(settings, 'DJANGO_CRADMIN_UICONTAINER_VALIDATE_BEM', True)

    def validate_dom_id(self, dom_id):
        if dom_id is False:
            return
        if not self.should_validate_dom_id():
            return
        normalized_dom_id = dom_id.replace('-', '').lower()
        if not dom_id.startswith('id_') or dom_id != normalized_dom_id:
            raise InvalidDomIdError(
                'dom_id must begin with "id_", be all lowercase, and can not contain "-". '
                '{dom_id!r} does not match this requirement.'.format(
                    dom_id=dom_id))

    def validate_bem(self, bem_block, bem_element):
        if not self.should_validate_bem():
            return
        if bem_block and bem_element:
            raise InvalidBemError(
                'Can not specify both bem_element or bem_block. An '
                'HTML element is eighter a BEM block or a BEM element.')
        if bem_block:
            if '__' in bem_block:
                raise InvalidBemError(
                    '{bem_block} is not a valid BEM block name. '
                    'BEM blocks do not contain "__". Are you sure you '
                    'did not mean to use the bem_element kwarg?'.format(
                        bem_block=bem_block
                    ))
        elif bem_element:
            if '__' not in bem_element:
                raise InvalidBemError(
                    '{bem_element} is not a valid BEM element name. '
                    'BEM elements must contain "__". Are you sure you '
                    'did not mean to use the bem_block kwarg?'.format(
                        bem_element=bem_element
                    ))

    def get_full_class_path_as_string(self):
        """
        Get full class path as string.

        Useful for providing some extra information in exceptions.
        Normally this will be in a traceback, but when dealing with
        things rendered by a Django template, this information is not
        always included.
        """
        return '{}.{}'.format(self.__class__.__module__, self.__class__.__name__)

    def validate_html_tag(self, html_tag):
        if html_tag and self.supported_html_tags and html_tag not in self.supported_html_tags:
            raise UnsupportedHtmlTagError('Unsupported HTML tag for {classpath}: {html_tag}'.format(
                classpath=self.get_full_class_path_as_string(),
                html_tag=self._overridden_html_tag
            ))

    def get_default_html_tag(self):
        """
        Get the default HTML tag to wrap renderable in.

        Can be overriden by the ``html_tag`` kwarg for :meth:`.__init__`.

        Returns ``"div"`` by default.
        """
        return 'div'

    @property
    def html_tag(self):
        """
        Get the HTML tag for this container.
        """
        return self._overridden_html_tag or self.get_default_html_tag()

    @property
    def html_tag_supports_children(self):
        """
        Does the html tag support children?

        If this returns ``False``, we:

        - Do not render an end tag for the wrapper element.
        - Do not allow children to be added to the container.

        Should be overridden to return ``False`` if the :meth:`.get_default_html_tag`
        does not allow for children. Examples of this case is if the
        wrapper html tag i ``input`` or ``hr``.

        See also :meth:`.can_have_children`, which should be used if the HTML tag
        should have and end tag, but not children.

        Returns:
            boolean: True by default.
        """
        return True

    @property
    def can_have_children(self):
        """
        Can this container have children?

        If this returns ``False``, :meth:`.add_child` will raise
        :class:`.NotAllowedToAddChildrenError`.

        Returns:
            boolean: The return value from :meth:`.html_tag_supports_children` by default.
        """
        return self.html_tag_supports_children

    def get_default_role(self):
        """
        Get the default value for the role attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def role(self):
        """
        Get the value for the role attribute of the html element.

        You should not override this. Override :meth:`.get_default_role` instead.
        """
        return self._overridden_role or self.get_default_role()

    def get_default_dom_id(self):
        """
        Get the default value for the id attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def dom_id(self):
        """
        Get the value for the id attribute of the html element.

        You should not override this. Override :meth:`.get_default_dom_id` instead.
        """
        return self._overridden_dom_id or self.get_default_dom_id()

    def get_html_element_attributes(self):
        """
        Get HTML element attributes as a dict.

        The dict is parsed by :func:`django.forms.utils.flatatt`,
        so:

        - ``{'myattribute': True}`` results in ``myattribute`` (no value).
        - ``{'myattribute': False}`` results in the attribute beeing ignored (not included in the output).
        - ``{'myattribute': 'Some value'}`` results in the ``myattribute="Some value"``.

        If you override this method, *remember to call super* to get
        the attributes set in the superclass.
        """
        return {
            'role': self.role,
            'id': self.dom_id,
            'class': self.css_classes or False,  # Fall back to false to avoid class=""
        }

    @property
    def html_element_attributes_string(self):
        """
        Get :meth:`.get_html_element_attributes` + any attributes in
        the ``html_element_attributes`` kwarg for :meth:`.__init__`
        encoded as a string using :func:`django.forms.utils.flatatt`.
        """
        html_element_attributes = dict(self.get_html_element_attributes())
        if self._html_element_attributes:
            html_element_attributes.update(self._html_element_attributes)
        return flatatt(html_element_attributes)

    def get_default_css_classes_list(self):
        """
        Override this to provide a default list of css classes.

        The css classes specified here can be overridden using
        the ``css_classes_list`` kwarg for :meth:`.__init__`.
        """
        return []

    def get_default_bem_block_or_element(self):
        """
        Get the default BEM block or element.

        A HTML element is eighter a BEM block or a
        BEM element, so we have joined this into
        a single method.
        """
        return None

    def get_bem_block_or_element(self):
        """
        Get the BEM block or element.

        DO NOT OVERRIDE THIS METHOD.
        Override :meth:`.get_default_bem_block_or_element` instead.
        """
        return (self._overridden_bem_block_or_element or
                self.get_default_bem_block_or_element())

    def get_default_bem_variant_list(self):
        """
        Get the default BEM variants.

        The full CSS class of any variant in the list will
        be :meth:`.get_bem_block_or_element` with ``--`` and
        the variant appended, so if the bem block/element is
        ``"menu"``, and the variant is ``"expanded"``, the
        resulting css class will be ``"menu--expanded"``.
        """
        return []

    def get_bem_variant_list(self):
        """
        Get the list of BEM variants.

        DO NOT OVERRIDE THIS METHOD.
        Override :meth:`.get_default_bem_variant_list` instead.
        """
        return self._overridden_bem_variant_list or self.get_default_bem_variant_list()

    def get_bem_css_classes_list(self):
        """
        Get the BEM css classes as list.

        DO NOT OVERRIDE THIS METHOD.
        Override :meth:`.get_default_bem_block_or_element`
        and :meth:`.get_default_bem_variant_list` instead.
        """
        bem_block_or_element = self.get_bem_block_or_element()
        bem_css_classes = []
        if bem_block_or_element:
            bem_css_classes.append(bem_block_or_element)
            for variant in self.get_bem_variant_list():
                css_class = '{}--{}'.format(bem_block_or_element, variant)
                bem_css_classes.append(css_class)
        return bem_css_classes

    def get_css_classes_list(self):
        """
        DO NOT OVERRIDE THIS METHOD.

        Unlike with :class:`django_cradmin.renderable.AbstractRenderableWithCss`,
        you do not override this class to add your own css classes. Override
        :meth:`.get_default_css_classes_list`.

        This is because this method respects the ``css_classes_list`` kwarg
        for :meth:`.__init__`, and just falls back to :meth:`.get_default_css_classes_list`.
        So if you override this method, the ``css_classes_list`` kwarg will be useless.
        """
        css_classes_list = self.get_bem_css_classes_list()
        if self._overridden_css_classes_list:
            css_classes_list.extend(self._overridden_css_classes_list)
        else:
            css_classes_list.extend(self.get_default_css_classes_list())
        if self._extra_css_classes_list:
            css_classes_list.extend(self._extra_css_classes_list)
        return css_classes_list

    def get_default_test_css_class_suffixes_list(self):
        """
        Override this to provide a default list of css classes for unit tests.

        The css classes specified here can be overridden using
        the ``test_css_class_suffixes_list`` kwarg for :meth:`.__init__`.
        """
        return ['uicontainer-{}'.format(self.__class__.__name__.lower())]

    def get_test_css_class_suffixes_list(self):
        """
        DO NOT OVERRIDE THIS METHOD.

        Unlike with :class:`django_cradmin.renderable.AbstractRenderableWithCss`,
        you do not override this class to add your own test css classes. Override
        :meth:`.get_default_test_css_class_suffixes_list`.

        This is because this method respects the ``test_css_class_suffixes_list`` kwarg
        for :meth:`.__init__`, and just falls back to :meth:`.get_default_test_css_class_suffixes_list`.
        So if you override this method, the ``test_css_class_suffixes_list`` kwarg will be useless.
        """
        if self._overridden_test_css_class_suffixes_list:
            test_css_class_suffixes_list = self._overridden_test_css_class_suffixes_list
        else:
            test_css_class_suffixes_list = self.get_default_test_css_class_suffixes_list()
        return test_css_class_suffixes_list

    def bootstrap(self, parent=None):
        """
        Bootstrap the container.

        Must be called once on the top-level container
        in the tree of containers.

        Sets the provided parent as :attr:`.parent`.

        Updates the properties of all children (using dict update())
        with :attr:`.properties`.
        """
        if self._is_bootstrapped:
            raise AlreadyBootsrappedError('The container is already bootstrapped. Can not bootstrap '
                                          'the same container twice.')
        self.parent = parent
        if self.parent:
            self.properties.update(self.parent.properties)
        for child in self._virtual_childrenlist:
            child.bootstrap(parent=self)
        for child in self._childrenlist:
            child.bootstrap(parent=self)
        self._is_bootstrapped = True
        return self

    def prepopulate_children_list(self):
        """
        Pre-polulate the children list.

        This is called in :meth:`.__init__` before
        any children from the kwargs is added.

        Returns:
            list: An empty list by default, but you can override this
            in subclasses.
        """
        return []

    def prepopulate_virtual_children_list(self):
        """
        Pre-polulate the virtual children list.

        This is called in :meth:`.__init__` before
        any children from the kwargs is added, and before any children
        is :meth:`.prepopulate_children_list` is added.

        Returns:
            list: An empty list by default, but you can override this
            in subclasses.
        """
        return []

    def add_child(self, childcontainer):
        """
        Add a child to the container.

        Args:
            childcontainer: A :class:`.AbstractContainerRenderable` object.

        Returns:
            A reference to self. This means that you can chain calls to this method.
        """
        if self.can_have_children:
            self._childrenlist.append(childcontainer)
            if self._is_bootstrapped and not childcontainer._is_bootstrapped:
                childcontainer.bootstrap(parent=self)
        else:
            raise NotAllowedToAddChildrenError('{modulename}.{classname} can not have children'.format(
                modulename=self.__class__.__module__,
                classname=self.__class__.__name__
            ))
        return self

    def add_virtual_child(self, childcontainer):
        """
        Add a "virtual" child to the container.

        This child is not rendered as a child of the container automatically
        (that is left to the template rendering the container). But it
        inherits properties and is automatically bootstrapped just like a
        regular child.

        Args:
            childcontainer: A :class:`.AbstractContainerRenderable` object.
        Returns:
            A reference to self. This means that you can chain calls to this method.
        """
        if self.can_have_children:
            self._virtual_childrenlist.append(childcontainer)
            if self._is_bootstrapped and not childcontainer._is_bootstrapped:
                childcontainer.bootstrap(parent=self)
        return self

    def add_children(self, *childcontainers):
        """
        Add children to the container.

        Args:
            *childcontainers: Zero or more :class:`.AbstractContainerRenderable` objects.

        Returns:
            A reference to self. This means that you can chain calls to this method.
        """
        for childcontainer in childcontainers:
            self.add_child(childcontainer)
        return self

    def add_virtual_children(self, *childcontainers):
        """
        Add virtual children to the container.

        Args:
            *childcontainers: Zero or more :class:`.AbstractContainerRenderable` objects.

        Returns:
            A reference to self. This means that you can chain calls to this method.
        """
        for childcontainer in childcontainers:
            self.add_virtual_child(childcontainer)
        return self

    def iter_children(self):
        """
        Returns an iterator over the children of this container.

        The yielded children will be objects of :class:`.AbstractContainerRenderable`
        subclasses.
        """
        return iter(self._childrenlist)

    def iter_virtual_children(self):
        """
        Returns an iterator over the virtual children of this container.

        The yielded children will be objects of :class:`.AbstractContainerRenderable`
        subclasses.
        """
        return iter(self._virtual_childrenlist)

    def get_childcount(self):
        """
        Get the number of children in the container.
        """
        return len(self._childrenlist)

    def get_virtual_childcount(self):
        """
        Get the number of virtual children in the container.
        """
        return len(self._virtual_childrenlist)

    @property
    def should_render(self):
        """
        Should we render anything?

        Override this to make the :meth:`.render` to control
        if the container is rendered. If this returns ``False``,
        :meth:`.render` returns an empty string instead of
        rendering the template.

        Returns:
            bool: ``True`` by default, but subclasses can override this behavior.
        """
        return True

    def render(self, **kwargs):
        """
        Overrides :meth:`django_cradmin.renderable.AbstractRenderable.render`.
        The only change is that we return an empty string if
        :meth:`.should_render` returns ``False``. If it returns ``True``,
        we call the overriden method and returns the result.

        Args:
            **kwargs: Forwarded to the overridden method if it is called.
        """
        if not self._is_bootstrapped:
            raise NotBootsrappedError(
                'Can not render an AbstractContainerRenderable that has not been bootstrapped. '
                'Ensure you call bootsrap() on the top-level container in the container '
                'hierarchy before rendering. Class causing this issue: {classpath}'.format(
                    classpath=self.get_full_class_path_as_string()
                ))
        if self.should_render:
            return super(AbstractContainerRenderable, self).render(**kwargs)
        else:
            return ''


class Div(AbstractContainerRenderable):
    """
    Renders a ``<div>``.

    The only thing this class does is to override
    :meth:`django_cradmin.uicontainer.container.AbstractContainerRenderable.get_default_html_tag`
    and return ``"div"``.
    """
    def get_default_html_tag(self):
        return 'div'


class NoWrapperElement(AbstractContainerRenderable):
    """
    Renders children, but no wrapper HTML element.
    """
    template_name = 'django_cradmin/uicontainer/no_wrapper_element.django.html'



import htmls
from django import test
from django_cradmin import uicontainer


class TestParagraph(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Paragraph().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('p'))

    def test_no_text(self):
        container = uicontainer.semantic.Paragraph().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('p').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.Paragraph(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('p').alltext_normalized, 'test')


class TestLink(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Link().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('a'))

    def test_no_text(self):
        container = uicontainer.semantic.Link().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('a').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.Link(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('a').alltext_normalized, 'test')

    def test_no_href(self):
        container = uicontainer.semantic.Link().bootstrap()
        selector = htmls.S(container.render())
        self.assertFalse(selector.one('a').hasattribute('href'))

    def test_with_href(self):
        container = uicontainer.semantic.Link(href='http://example.com').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('a')['href'], 'http://example.com')


class TestH1(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.H1().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('h1'))

    def test_no_text(self):
        container = uicontainer.semantic.H1().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h1').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.H1(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h1').alltext_normalized, 'test')


class TestH2(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.H2().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('h2'))

    def test_no_text(self):
        container = uicontainer.semantic.H2().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h2').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.H2(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h2').alltext_normalized, 'test')


class TestH3(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.H3().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('h3'))

    def test_no_text(self):
        container = uicontainer.semantic.H3().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h3').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.H3(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h3').alltext_normalized, 'test')


class TestH4(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.H4().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('h4'))

    def test_no_text(self):
        container = uicontainer.semantic.H4().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h4').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.H4(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h4').alltext_normalized, 'test')


class TestSection(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Section().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('section'))


class TestHeader(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Header().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('header'))


class TestFooter(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Footer().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('footer'))


class TestMain(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Main().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('main'))

    def test_role(self):
        container = uicontainer.semantic.Main().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('main')['role'], 'main')


class TestNav(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Nav().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('nav'))


class TestUl(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Ul().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('ul'))


class TestOl(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Ol().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('ol'))


class TestLi(test.TestCase):
    def test_sanity(self):
        container = uicontainer.semantic.Li().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('li'))

    def test_no_text(self):
        container = uicontainer.semantic.Li().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('li').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.semantic.Li(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('li').alltext_normalized, 'test')



from __future__ import unicode_literals

from django.test import TestCase
from django_cradmin.python2_compatibility import mock
from model_mommy import mommy
from future import standard_library

from django_cradmin.tests.test_viewhelpers.cradmin_viewhelpers_testapp.models import FilterTestModel
from django_cradmin.viewhelpers.listfilter.base.abstractfilter import AbstractFilter
from django_cradmin.viewhelpers.listfilter.base.exceptions import InvalidFiltersStringError
from django_cradmin.viewhelpers.listfilter.base.filtershandler import FiltersHandler

standard_library.install_aliases()


class MinimalIntFilter(AbstractFilter):
    def get_slug(self):
        return 'i'

    def clean_value(self, value):
        return int(value)


class MinimalStringFilter(AbstractFilter):
    def get_slug(self):
        return 's'


class TestFiltersHandler(TestCase):
    def test_split_raw_filter_values(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        self.assertEqual(
            ['a/b', 'c,d'],
            filtershandler.split_raw_filter_values('a%252Fb%2Cc%252Cd'))

    def test_join_filter_values(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        self.assertEqual(
            'a%252Fb%2Cc%252Cd',
            filtershandler.join_filter_values(['a/b', 'c,d']))

    def test_invalid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        with self.assertRaisesMessage(InvalidFiltersStringError,
                                      '"x" does not contain "-".'):
            filtershandler.parse('x')

    def test_invalid_filter_slug(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        with self.assertRaisesMessage(InvalidFiltersStringError,
                                      '"x" is not a valid filter slug.'):
            filtershandler.parse('x-10')

    def test_add_filter_duplicate_filter_slug(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(AbstractFilter(slug='x'))
        with self.assertRaisesMessage(ValueError,
                                      'Duplicate slug: "x".'):
            filtershandler.add_filter(AbstractFilter(slug='x'))

    def test_add_filter_invalid_filter_slug(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        with self.assertRaisesMessage(ValueError,
                                      'Invalid filter slug: "x-y". Slugs can not contain "-".'):
            filtershandler.add_filter(AbstractFilter(slug='x-y'))

    def test_parse_empty_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.parse('')
        self.assertEqual(
            {},
            filtershandler.filtermap)

    def test_parse_none(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.parse(None)
        self.assertEqual(
            {},
            filtershandler.filtermap)

    def test_parse_simple_valid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse('s-test')
        self.assertEqual(
            ['test'],
            filtershandler.filtermap['s'].values)

    def test_parse_multivalue_valid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse('s-a,b,c')
        self.assertEqual(
            ['a', 'b', 'c'],
            filtershandler.filtermap['s'].values)

    def test_parse_simple_valid_filter_string_trailing_slash(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse('s-test/')
        self.assertEqual(
            ['test'],
            filtershandler.filtermap['s'].values)

    def test_parse_simple_valid_filter_string_leading_slash(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.parse('/s-test')
        self.assertEqual(
            ['test'],
            filtershandler.filtermap['s'].values)

    def test_parse_complex_valid_filter_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalStringFilter())
        filtershandler.add_filter(MinimalIntFilter())
        filtershandler.parse('/i-10/s-jane,joe/')
        self.assertEqual(2, len(filtershandler.filtermap))
        self.assertEqual(
            ['10'],
            filtershandler.filtermap['i'].values)
        self.assertEqual(
            ['jane', 'joe'],
            filtershandler.filtermap['s'].values)

    def test_build_filters_string(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=['10'])
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=['jane', 'joe'])
        filtershandler.add_filter(intfilter)
        filtershandler.add_filter(stringfilter)

        new_stringfilter = stringfilter.copy()
        new_stringfilter.set_values(values=['jack', 'peter'])
        self.assertEqual(
            'i-10/s-jack%2Cpeter',
            filtershandler.build_filters_string(changed_filterobject=new_stringfilter))

    def test_build_filter_url(self):
        def urlbuilder(filters_string):
            return '/the/prefix/{}?a=querystring'.format(filters_string)

        filtershandler = FiltersHandler(urlbuilder=urlbuilder)
        intfilter = MinimalIntFilter()
        intfilter.set_values(values=['10'])
        stringfilter = MinimalStringFilter()
        stringfilter.set_values(values=['jane', 'joe'])
        filtershandler.add_filter(intfilter)
        filtershandler.add_filter(stringfilter)

        new_stringfilter = stringfilter.copy()
        new_stringfilter.set_values(values=['jack', 'peter'])
        self.assertEqual(
            '/the/prefix/i-10/s-jack%2Cpeter?a=querystring',
            filtershandler.build_filter_url(changed_filterobject=new_stringfilter))

    def test_filter(self):
        class FilterOne(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mycharfield='test')

        class FilterTwo(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mybooleanfield=True)

        match = mommy.make('cradmin_viewhelpers_testapp.FilterTestModel',
                           mycharfield='test', mybooleanfield=True)
        mommy.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield='no match', mybooleanfield=True)
        mommy.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield='test', mybooleanfield=False)

        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(FilterOne(slug='filterone'))
        filtershandler.add_filter(FilterTwo(slug='filtertwo'))
        queryset = filtershandler.filter(FilterTestModel.objects.all())
        self.assertEqual({match},
                         set(queryset))

    def test_filter_exclude(self):
        class FilterOne(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mycharfield='test')

        class FilterTwo(AbstractFilter):
            def filter(self, queryobject):
                return queryobject.filter(mybooleanfield=True)

        match1 = mommy.make('cradmin_viewhelpers_testapp.FilterTestModel',
                            mycharfield='test', mybooleanfield=True)
        match2 = mommy.make('cradmin_viewhelpers_testapp.FilterTestModel',
                            mycharfield='no match', mybooleanfield=True)
        mommy.make('cradmin_viewhelpers_testapp.FilterTestModel',
                   mycharfield='test', mybooleanfield=False)

        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(FilterOne(slug='filterone'))
        filtershandler.add_filter(FilterTwo(slug='filtertwo'))
        queryset = filtershandler.filter(FilterTestModel.objects.all(),
                                         exclude={'filterone'})
        self.assertEqual({match1, match2},
                         set(queryset))

    def test_get_label_for(self):
        class MyFilter(AbstractFilter):
            pass

        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MyFilter(label='Test Label', slug='test'))
        self.assertEqual(
            'Test Label',
            filtershandler.get_label_for('test'))

    def test_get_cleaned_value_for(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalIntFilter())
        filtershandler.parse('/i-10')
        self.assertEqual(
            10,
            filtershandler.get_cleaned_value_for('i'))

    def test_get_cleaned_values_for(self):
        filtershandler = FiltersHandler(urlbuilder=mock.MagicMock())
        filtershandler.add_filter(MinimalIntFilter())
        filtershandler.parse('/i-10')
        self.assertEqual(
            [10],
            filtershandler.get_cleaned_values_for('i'))



class DjangoOrmFilterMixin(object):
    """
    Mixin class for Django ORM filters.
    """

    def get_modelfield(self):
        """
        Get the name of the model field to filter on.

        This may not make sense for all filters, but it is a fairly
        common thing to need.

        Defaults to :meth:`django_cradmin.viewhelpers.listfilter.base.AbstractFilter.get_slug`.
        """
        return self.get_slug()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
from collections import OrderedDict

import pythonkss
from django.conf import settings
from django.templatetags.static import static
from ievv_opensource.utils import ievvbuildstatic
from ievv_opensource.utils.singleton import Singleton


class AbstractStyleGuide(object):
    #: The extensions to look for when finding style files.
    stylefile_extensions = ['.less', '.css', '.sass', '.scss']

    def __init__(self, unique_id, label,
                 template_name="cradmin_kss_styleguide/styleguideview/guide.django.html",
                 section_template_name='cradmin_kss_styleguide/templatetags/render_kss_section.django.html',
                 sections_template_name='cradmin_kss_styleguide/templatetags/render_kss_sections.django.html',
                 toc_template_name='cradmin_kss_styleguide/templatetags/render_kss_toc.django.html',
                 toc_node_template_name='cradmin_kss_styleguide/templatetags/render_kss_toc_node.django.html',
                 example_template_name='cradmin_kss_styleguide/styleguideview/example.django.html',
                 frontpage_template=None,
                 filename_patterns=None):
        """
        Args:
            unique_id: A unique ID for the styleguide.
            label: The label of the styleguide.
        """
        self.unique_id = unique_id
        self.label = label
        self.template_name = template_name
        self.section_template_name = section_template_name
        self.sections_template_name = sections_template_name
        self.toc_template_name = toc_template_name
        self.toc_node_template_name = toc_node_template_name
        self.frontpage_template = frontpage_template
        self.example_template_name = example_template_name
        self.javascript_component_ids = self.get_javascript_component_ids()
        self.filename_patterns = filename_patterns

    def get_sourcefolders(self):
        raise NotImplementedError()

    def get_cssurl_for_styleguide(self):
        raise NotImplementedError()

    def make_kss_styleguide(self):
        return pythonkss.Parser(
            *self.get_sourcefolders(),
            extensions=self.stylefile_extensions,
            filename_patterns=self.filename_patterns)

    def get_template_name(self):
        return self.template_name

    def get_section_template_name(self):
        return self.section_template_name

    def get_sections_template_name(self):
        return self.sections_template_name

    def get_toc_template_name(self):
        return self.toc_template_name

    def get_toc_node_template_name(self):
        return self.toc_node_template_name

    def get_example_template_name(self):
        return self.example_template_name

    def __str__(self):
        return self.label

    def get_javascript_component_ids(self):
        return []


class StyleGuide(AbstractStyleGuide):
    def __init__(self, sourcefolders, cssurl_for_styleguide, **kwargs):
        """
        Args:
            sourcefolders (list): List of source folders.
            **kwargs: See :class:`.AbstractStyleGuide`.
        """
        super(StyleGuide, self).__init__(**kwargs)
        self.sourcefolders = sourcefolders
        self.cssurl_for_styleguide = cssurl_for_styleguide

    def get_sourcefolders(self):
        return self.sourcefolders

    def get_cssurl_for_styleguide(self):
        return self.cssurl_for_styleguide


class IevvBuildstaticStyleGuide(AbstractStyleGuide):
    def __init__(self, appname, sourcefile, sourcefolder='styles', **kwargs):
        super(IevvBuildstaticStyleGuide, self).__init__(**kwargs)
        self.appname = appname
        self.sourcefolder = sourcefolder
        self.sourcefile = sourcefile

    def find_plugin(self):
        app = settings.IEVVTASKS_BUILDSTATIC_APPS.get_app(self.appname)
        for plugin in app.plugins:
            if isinstance(plugin, ievvbuildstatic.sassbuild.Plugin):
                if plugin.sourcefolder == self.sourcefolder and plugin.sourcefile == self.sourcefile:
                    return plugin
        raise ValueError(
            'Plugin in {appname} matching sourcefolder={sourcefolder} and '
            'sourcefile={sourcefile} not found'.format(
                appname=self.appname,
                sourcefolder=self.sourcefolder,
                sourcefile=self.sourcefile))

    def get_sourcefolders(self):
        plugin = self.find_plugin()
        sourcefolders = [plugin.get_sourcefolder_path()]
        sourcefolders.extend(plugin.get_other_sourcefolders_paths())
        return sourcefolders

    def get_cssurl_for_styleguide(self):
        plugin = self.find_plugin()
        staticurl = plugin.get_destinationfile_path().split('{sep}static{sep}'.format(sep=os.sep))[1]
        return static(staticurl)


class CradminStyleGuide(IevvBuildstaticStyleGuide):
    def __init__(self, *args, **kwargs):
        template_name = kwargs.pop(
            'template_name',
            'cradmin_kss_styleguide/styleguideview/cradmin-guide.django.html')
        example_template_name = kwargs.pop(
            'example_template_name',
            'cradmin_kss_styleguide/styleguideview/cradmin-example.django.html')
        kwargs['template_name'] = template_name
        kwargs['example_template_name'] = example_template_name
        super(CradminStyleGuide, self).__init__(*args, **kwargs)

    def get_javascript_component_ids(self):
        return ['django_cradmin_javascript']


class Registry(Singleton):
    """
    Registry of :class:`.StyleGuide` objects.

    Examples:

        First, define a subclass of :class:`.AbstractCustomSql`.

        Register a StyleGuide in the registry via an AppConfig for your
        Django app::

            from django.apps import AppConfig
            from django_cradmin.apps.cradmin_kss_styleguide import styleguide_registry

            class MyAppConfig(AppConfig):
                name = 'myapp'

                def ready(self):
                    styleguide = styleguide_registry.StyleGuide(
                        unique_id='my_styles',
                        label='My styles',
                        source_folder='/path/to/my/styles/')
                    styleguide_registry.Registry.get_instance().add(styleguide)
    """

    def __init__(self):
        super(Registry, self).__init__()
        self._styleguides = OrderedDict()

    def add(self, styleguide):
        """
        Add the given ``customsql_class`` to the registry.

        Parameters:
            styleguide (.AbstractStyleGuide): The styleguide.
        """
        if styleguide.unique_id in self._styleguides:
            raise KeyError('Duplicate unique_id in styleguide: {}'.format(styleguide.unique_id))
        self._styleguides[styleguide.unique_id] = styleguide

    def __getitem__(self, unique_id):
        """
        Get the :class:`.AbstractStyleGuide` registered with ``unique_id``.
        """
        return self._styleguides[unique_id]

    def __len__(self):
        return len(self._styleguides)

    def first_guide(self):
        return list(self.iterstyleguides())[0]

    def iterstyleguides(self):
        return self._styleguides.values()


class MockableRegistry(Registry):
    """
    A non-singleton version of :class:`.Registry`. For tests.

    Typical usage in a test::

        from django_cradmin.apps.cradmin_kss_styleguide import styleguide_registry

        class MockStyleGuide(styleguide_registry.StyleGuide):
            # ...

        mockregistry = styleguide_registry.MockableRegistry()
        mockregistry.add(MockStyleGuide(...))

        with mock.patch('django_cradmin.apps.cradmin_kss_styleguide.styleguide_registry.Registry.get_instance',
                        lambda: mockregistry):
            pass  # ... your code here ...
    """

    def __init__(self):
        self._instance = None  # Ensure the singleton-check is not triggered
        super(MockableRegistry, self).__init__()



class PreviewMixin:
    """
    Mixin class to provide a preview view for your views.

    You must override :meth:`.serialize_preview`, :meth:`.deserialize_preview` and :meth:`.get_preview_url`,
    and you must call :meth:`.store_preview_in_session` before opening the preview.

    You must also ensure your template extends ``django_cradmin/viewhelpers/formview_base.django.html``.

    Preview data is added to a Django session using :meth:`.store_preview_in_session`,
    and popped (fetched and removed) from the session in the preview view using
    :meth:`get_preview_data`. A typical place to call :meth:`.store_preview_in_session`
    is in the form_valid() method of form views. Example::

        class MyFormView(formbase.PreviewMixin, formbase.FormView):

            # Ensure this extends django_cradmin/viewhelpers/formview_base.django.html
            template_name = 'myapp/mytemplate.django.html'


            # ... other required code for formbase.FormView ...


            def form_valid(self, form):
                if self.preview_requested():
                    self.store_preview_in_session(self.serialize_preview(form))
                    return self.render_to_response(self.get_context_data(form=form, show_preview=True))
                else:
                    # ... save, update, or whatever you do on POST when preview is not requested ...

            def get_buttons(self):
                return [
                    PrimarySubmit('save', _('Save')),

                    # When this button is clicked, self.preview_requested() returns True (see form_valid above).
                    DefaultSubmit(self.submit_preview_name, _('Preview'))
                ]

            def serialize_preview(self, form):
                return json.dumps({
                    'title': form.cleaned_data['title'],
                    'description': form.cleaned_data['description'],
                })

            @classmethod
            def deserialize_preview(cls, serialized):
                return json.loads(serialized)


    If you have something like MyFormView implemented, a preview view is as simple as this::

        class MyPreviewView(View):
            def get(request):
                preview_data = MyFormView.get_preview_data()
                return HttpResponse(...)

    How you render your preview data in your view is entirely up to you - a TemplateView
    that fetches preview data in get_context_data() is ususally more approproate than a
    View like the example above.
    """

    #: The name of the submit button used for preview.
    submit_preview_name = 'submit-preview'

    def preview_requested(self):
        """
        Determine if a preview was requested.

        Defaults to checking if :obj:`.submit_preview_name` is
        in ``request.POST``.
        """
        return self.submit_preview_name in self.request.POST

    def store_preview_in_session(self, data):
        self.request.session[self.__class__.get_preview_sessionkey()] = data

    def serialize_preview(self, form):
        """
        Seralize data for preview.

        You must override this and :meth:`.deserialize_preview` - they work together
        to send the preview to the preview View. You can return anything that can
        be put into a Django session here. We recommend returning a string to
        ensure your code work with any session backend. JSON encoding is a good choice
        is most cases.
        """
        raise NotImplementedError()

    @classmethod
    def deserialize_preview(cls, serialized):
        """
        Deseralize a preview serialized with :meth:`.serialize_preview`.

        You must override this and :meth:`.serialize_preview` - they work together
        to send the preview to the preview View.
        """
        raise NotImplementedError()

    @classmethod
    def get_preview_sessionkey(cls):
        """
        Get the session key used for preview. You should not
        need to override this.
        """
        sessionkey = 'django_cradmin__{module}.{classname}'.format(
            module=cls.__module__,
            classname=cls.__name__)
        return sessionkey

    @classmethod
    def get_preview_data(cls, request):
        """
        Get the preview data.

        You should use this in the preview view to get the
        data for the preview.

        You normally do not override this. If you want to manage
        serialization yourself, see :meth:`.serialize_preview`.
        """
        serialized = request.session.pop(cls.get_preview_sessionkey())
        return cls.deserialize_preview(serialized)

    def get_preview_url(self):
        """
        Get the URL of the preview view.
        """
        return None

    def add_preview_mixin_context_data(self, context):
        """
        Must be used by get_context_data() in subclasses to add the context data required to
        render the view.

        Examples:

            Adding the required context data::

                def get_context_data(self, **kwargs):
                    context = super(MyView, self).get_context_data(**kwargs)
                    self.add_preview_mixin_context_data(context=context)
                    return context
        """
        if context.get('show_preview', False):
            context['preview_url'] = self.get_preview_url()
            context['show_preview'] = True



import htmls
from django import test

from django_cradmin import python2_compatibility
from django_cradmin.viewhelpers.multiselect2 import selected_item_renderer
from django_cradmin.python2_compatibility import mock


class TestSelectedItem(test.TestCase):
    def test_title(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = 'testvalue'
        selector = htmls.S(selected_item_renderer.SelectedItem(
            value=mockvalue).render())
        self.assertEqual(
            'testvalue',
            selector.one('.django-cradmin-multiselect2-target-selected-item-title').alltext_normalized)

    def test_no_description(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(selected_item_renderer.SelectedItem(
            value=mockvalue).render())
        self.assertFalse(
            selector.exists('.django-cradmin-multiselect2-target-selected-item-description'))

    def test_has_description(self):
        class MySelectedItem(selected_item_renderer.SelectedItem):
            def get_description(self):
                return 'test description'

        mockvalue = mock.MagicMock()
        selector = htmls.S(MySelectedItem(value=mockvalue).render())
        self.assertEqual(
            'test description',
            selector.one('.django-cradmin-multiselect2-target-selected-item-description').alltext_normalized)

    def test_input_field(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = 10
        selector = htmls.S(selected_item_renderer.SelectedItem(
            value=mockvalue).render())
        self.assertEqual(
            '10',
            selector.one('input[name="selected_items"]')['value'])

    def test_custom_input_field_name(self):
        class MySelectedItem(selected_item_renderer.SelectedItem):
            def get_inputfield_name(self):
                return 'testfield'

        mockvalue = mock.MagicMock()
        selector = htmls.S(MySelectedItem(value=mockvalue).render())
        self.assertTrue(selector.exists('input[name="testfield"]'))

    def test_deselectbutton_text(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(selected_item_renderer.SelectedItem(
            value=mockvalue).render())
        self.assertEqual(
            'Deselect',
            selector.one('button').alltext_normalized)

    def test_deselectbutton_aria_label(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = 'testvalue'
        selector = htmls.S(selected_item_renderer.SelectedItem(
            value=mockvalue).render())
        self.assertEqual(
            'Deselect "testvalue"',
            selector.one('button')['aria-label'])



from django import test
from django_cradmin import uicontainer


class TestEscapedText(test.TestCase):
    def test_no_text(self):
        container = uicontainer.text.EscapedText(text='').bootstrap()
        output = container.render()
        self.assertEqual(output, '\n')

    def test_simple_text(self):
        container = uicontainer.text.EscapedText(text='test').bootstrap()
        output = container.render()
        self.assertEqual(output, 'test\n')

    def test_html_text(self):
        container = uicontainer.text.EscapedText(text='<p>Hello</p>').bootstrap()
        output = container.render()
        self.assertEqual(output, '&lt;p&gt;Hello&lt;/p&gt;\n')


class TestHtml(test.TestCase):
    def test_no_html(self):
        container = uicontainer.text.Html(html='').bootstrap()
        output = container.render()
        self.assertEqual(output.strip(), '')

    def test_html(self):
        container = uicontainer.text.Html(html='<p>Hello</p>').bootstrap()
        output = container.render()
        self.assertEqual(output.strip(), '<p>Hello</p>')

    def test_get_default_html(self):
        class CustomHtml(uicontainer.text.Html):
            def get_default_html(self):
                return 'test'

        container = CustomHtml().bootstrap()
        output = container.render()
        self.assertEqual(output.strip(), 'test')

    def test_get_default_html_overridden_by_kwarg(self):
        class CustomHtml(uicontainer.text.Html):
            def get_default_html(self):
                return 'test'

        container = CustomHtml(html='overridden').bootstrap()
        output = container.render()
        self.assertEqual(output.strip(), 'overridden')



# from __future__ import unicode_literals
#
# from django import forms
# from django import test
# from . import mixins
#
#
# class TestFieldWrapperRenderable(test.TestCase, mixins.SingleFormRenderableHelperMixin):
#     def test_stuff(self):
#         selector = self.single_field_formrenderable_htmls(field=forms.CharField())
#         selector.prettyprint()



from django_cradmin import renderable


class AbstractHeaderRenderable(renderable.AbstractRenderableWithCss):
    def __init__(self, cradmin_instance):
        self.cradmin_instance = cradmin_instance

    def get_wrapper_htmltag(self):
        """
        Get the HTML tag to wrap the header in.

        Defaults to ``"header"``.
        """
        return 'header'

    def get_wrapper_htmltag_id(self):
        return 'id_django_cradmin_page_header'

    def include_menutoggle_javascript(self):
        return False


class DefaultHeaderRenderable(AbstractHeaderRenderable):
    template_name = 'django_cradmin/crheader/default-header.django.html'

    @property
    def bem_block(self):
        return 'adminui-page-header'

    @property
    def bem_variants_list(self):
        return []

    def expand_bem_variants_list(self):
        bem_block = self.bem_block
        return ['{}--{}'.format(bem_block, variant) for variant in self.bem_variants_list]

    def get_base_css_classes_list(self):
        return [self.bem_block] + self.expand_bem_variants_list()

    def include_menutoggle_javascript(self):
        return False



from . import container


class AdminuiPageSection(container.AbstractContainerRenderable):
    """
    ``adminui-page-section`` BEM block.
    """
    def get_default_bem_block_or_element(self):
        return 'adminui-page-section'

    def get_default_test_css_class_suffixes_list(self):
        return ['adminui-page-section']


class AdminuiPageSectionTight(AdminuiPageSection):
    """
    ``adminui-page-section`` BEM block with the ``tight`` BEM variant.
    """
    def get_default_bem_variant_list(self):
        return ['tight']


class Container(container.AbstractContainerRenderable):
    """
    ``container`` BEM block.
    """
    def get_default_bem_block_or_element(self):
        return 'container'

    def get_default_test_css_class_suffixes_list(self):
        return ['container']


class ContainerTight(Container):
    """
    ``container`` BEM block with the ``tight`` BEM variant.
    """
    def get_default_bem_variant_list(self):
        return ['tight']



from django_cradmin import renderable


class AbstractFooter(renderable.AbstractRenderableWithCss):
    def __init__(self, cradmin_instance):
        self.cradmin_instance = cradmin_instance

    def get_wrapper_htmltag(self):
        """
        Get the HTML tag to wrap the footer in.

        Defaults to ``"footer"``.
        """
        return 'footer'



from django_cradmin.uicontainer import convenience


class Button(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<button type="button">`` with the default
    style (using the ``button`` css class).
    """
    def __init__(self, button_type=None, name=False, **kwargs):
        self._overridden_button_type = button_type
        self._overridden_name = name
        super(Button, self).__init__(**kwargs)

    def get_default_html_tag(self):
        return 'button'

    def get_default_button_type(self):
        """
        Get the default value for the type attribute of the html element.

        Defaults to ``"button"``.
        """
        return 'button'

    @property
    def button_type(self):
        """
        Get the value for the type attribute of the html element.

        You should not override this. Override :meth:`.get_default_button_type` instead.
        """
        return self._overridden_button_type or self.get_default_button_type()

    def get_default_name(self):
        """
        Get the default value for the name attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def name(self):
        """
        Get the value for the name attribute of the button html element.

        You should not override this. Override :meth:`.get_default_name` instead.
        """
        return self._overridden_name or self.get_default_name()

    def get_html_element_attributes(self):
        html_attributes = super(Button, self).get_html_element_attributes()
        html_attributes['type'] = self.button_type
        html_attributes['name'] = self.name
        return html_attributes

    def get_bem_block_or_element(self):
        return 'button'


class ButtonPrimary(Button):
    """
    Renders a ``<button type="button">`` with the primary
    style (using the ``button  button--primary`` css class).
    """
    def get_default_bem_variant_list(self):
        return ['primary']


class ButtonHistoryBack(Button):
    """
    Renders a ``<button type="button">`` just like
    :class:`.Button`, with ``onclick="history.back();return false;"``.

    Typically used for cancel buttons where we just want to return
    to the previous page.
    """
    def get_html_element_attributes(self):
        attributes = super(ButtonHistoryBack, self).get_html_element_attributes()
        attributes['onclick'] = 'history.back();return false;'
        return attributes


class Submit(Button):
    """
    Renders a ``<button type="submit">`` with the default
    style (using the ``button`` css class).
    """
    def get_default_button_type(self):
        return 'submit'

    def get_default_test_css_class_suffixes_list(self):
        return ['submit']


class SubmitPrimary(Submit):
    """
    Renders a ``<button type="submit">`` with the primary
    style (using the ``button  button--primary`` css class).
    """
    def get_default_bem_variant_list(self):
        return ['primary']

    def get_default_test_css_class_suffixes_list(self):
        return ['submit-primary']



from __future__ import unicode_literals
from django.views.generic import TemplateView

from django_cradmin import javascriptregistry


class EmailSentView(TemplateView, javascriptregistry.viewmixin.StandaloneBaseViewMixin):
    template_name = 'cradmin_register_account/email_sent.django.html'

    def get_context_data(self, **kwargs):
        context = super(EmailSentView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context



from .previewmixin import PreviewMixin  # noqa
from .formviewmixin import FormViewMixin  # noqa
from .formview import WithinRoleFormView  # noqa
from .formview import StandaloneFormView  # noqa
from .create_update_view_mixin import CreateUpdateViewMixin  # noqa
from .createview import WithinRoleCreateView  # noqa
from .updateview import WithinRoleUpdateView  # noqa
from .updateview import UpdateRoleView  # noqa
from .deleteview import WithinRoleDeleteView  # noqa



from django.test import TestCase
from model_mommy import mommy

from django_cradmin.cradmin_testhelpers import TestCaseMixin
from django_cradmin.viewhelpers import listbuilderview
from django_cradmin.django_cradmin_testapp import models as testmodels


class ListBuilderViewWithoutPaging(listbuilderview.View):
    model = testmodels.SomeItem

    def get_queryset_for_role(self):
        return testmodels.SomeItem.objects.all()


class ListBuilderViewWithPaging(ListBuilderViewWithoutPaging):
    paginate_by = 3


class TestListBuilderView(TestCase, TestCaseMixin):
    viewclass = ListBuilderViewWithoutPaging

    def test_empty(self):
        mockresponse = self.mock_http200_getrequest_htmls()
        self.assertFalse(mockresponse.selector.exists('.test-cradmin-listbuilder-list'))
        self.assertEquals(
            mockresponse.selector.one('.test-cradmin-no-items-message').alltext_normalized,
            'No some items')

    def test_not_empty(self):
        mommy.make('django_cradmin_testapp.SomeItem',
                   name='Test name')
        mockresponse = self.mock_http200_getrequest_htmls()
        self.assertFalse(mockresponse.selector.exists('.test-cradmin-no-items-message'))
        self.assertTrue(mockresponse.selector.exists('.test-cradmin-listbuilder-list'))

    def test_item_rendering(self):
        mommy.make('django_cradmin_testapp.SomeItem',
                   name='Test name')
        mockresponse = self.mock_http200_getrequest_htmls()
        # mockresponse.selector.prettyprint()
        self.assertEqual(1, mockresponse.selector.count('.test-cradmin-listbuilder-item-frame-renderer'))
        self.assertEqual('Test name',
                         mockresponse.selector.one('.test-cradmin-listbuilder-item-frame-renderer').alltext_normalized)


class TestListBuilderPaginationView(TestCase, TestCaseMixin):
    viewclass = ListBuilderViewWithPaging

    def test_paginate_by_singlepage(self):
        mommy.make('django_cradmin_testapp.SomeItem', _quantity=3)
        mockresponse = self.mock_http200_getrequest_htmls()
        self.assertEqual(3, mockresponse.selector.count('.test-cradmin-listbuilder-item-frame-renderer'))
        self.assertFalse(mockresponse.selector.exists('.django-cradmin-loadmorepager'))

    def test_paginate_by_firstpage(self):
        mommy.make('django_cradmin_testapp.SomeItem', _quantity=8)
        mockresponse = self.mock_http200_getrequest_htmls()
        # mockresponse.selector.one('#django_cradmin_contentwrapper').prettyprint()
        self.assertEqual(3, mockresponse.selector.count('.test-cradmin-listbuilder-item-frame-renderer'))
        self.assertTrue(mockresponse.selector.exists('.django-cradmin-loadmorepager'))

    def test_paginate_by_middlepage(self):
        mommy.make('django_cradmin_testapp.SomeItem', _quantity=8)
        mockresponse = self.mock_http200_getrequest_htmls(requestkwargs={'data': {'page': 2}})
        self.assertEqual(3, mockresponse.selector.count('.test-cradmin-listbuilder-item-frame-renderer'))
        self.assertTrue(mockresponse.selector.exists('.django-cradmin-loadmorepager'))

    def test_paginate_by_lastpage(self):
        mommy.make('django_cradmin_testapp.SomeItem', _quantity=8)
        mockresponse = self.mock_http200_getrequest_htmls(requestkwargs={'data': {'page': 3}})
        self.assertEqual(2, mockresponse.selector.count('.test-cradmin-listbuilder-item-frame-renderer'))
        self.assertFalse(mockresponse.selector.exists('.django-cradmin-loadmorepager'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json

import django_cradmin
import re

from django.contrib.staticfiles.templatetags import staticfiles
from django.template.loader import render_to_string

COMPONENT_ID_REGEX = re.compile(r'^[a-z][a-z0-9_]*[a-z0-9]$')


class ComponentIdFormatError(Exception):
    """
    Raised by :class:`.AbstractJsComponent` if the component_id is
    not formatted according to the rules.
    """


class AbstractJsComponent(object):
    """
    Base class for all javascript components.

    .. attribute:: component_id

        Set to :meth:`.get_component_id` in __init__ if the component ID passes
        the validation checks.
    """
    @classmethod
    def get_component_id(cls):
        """
        Get a unique ID for this javascript component.

        Can only contain a-z, 0-9 and ``_``. The first letter can only be a-z,
        and the last letter can not be ``_``.

        Must be overridden in subclasses.
        """
        raise NotImplementedError()

    def __init__(self, request):
        self.request = request
        self.component_id = self.get_component_id()
        if not COMPONENT_ID_REGEX.match(self.component_id):
            raise ComponentIdFormatError(
                'Invalid component_id: {}. Must start with a-z, end with a-z or 0-9, and'
                'the letters between the start and end can only be a-z, 0-9 and _. Must '
                'be at least 2 letters.'.format(
                    self.component_id))

    def get_dependencies(self):
        """
        Get a list of :meth:`component ids <.get_component_id>` that this javascript
        component depends on.
        """
        return []

    def get_static_url(self, path):
        """
        Works just like the ``static`` template tag.

        Args:
            path (str): The path relative to the static directory.

        Returns:
            str: The static URL.

        """
        return staticfiles.static(path)

    def get_sourceurls(self):
        """
        Get a list of javascript source files to load for this component.

        These URLs are added using ``<script>`` tags at the end of ``<body>``
        by the base cradmin templates.

        .. note:: If other components load the same source files, the template tags
            in the ``cradmin_javascriptregistry_tags`` template tag library
            ensures that one unique sources are loaded.
        """
        return []

    def get_head_sourceurls(self):
        """
        Just like :meth:`.get_sourceurls`, but added in the ``<head>`` tag.

        You should normally avoid using this because adding scripts to the
        head tag blocks page loading.
        """
        return []

    def get_javascript_code_after_sourceurls(self):
        """
        Javascript code to add inline in the body of the HTML page
        after loading the source URLs (see :meth:`.get_sourceurls`).

        See :meth:`.get_javascript_code_before_sourceurls` for more details.
        """
        return None

    def get_javascript_code_before_sourceurls(self):
        """
        Javascript code to add inline in the body of the HTML page
        before loading the source URLs (see :meth:`.get_sourceurls`).

        Should return javascript code as a string (no HTML tags).
        """
        return None

    def __eq__(self, other):
        """
        Javascript components are considered equal if their :attr:`~.AbstractJsComponent.component_id` match.
        """
        return other.component_id == self.component_id


class CradminAngular1(AbstractJsComponent):
    def get_target_domelement_selector(self):
        raise NotImplementedError()

    def get_head_sourceurls(self):
        return [
            self.get_static_url('django_cradmin/dist/vendor/cradmin-vendorjs.js'),
            self.get_static_url('django_cradmin/dist/js/cradmin.min.js'),
        ]

    def get_angularjs_modules(self):
        return []

    def get_angularjs_modules_json(self):
        return json.dumps(self.get_angularjs_modules())

    def get_javascript_code_after_sourceurls(self):
        return render_to_string('django_cradmin/utils/jsregistry/cradmin-angular1-jscomponent.django.js',
                                request=self.request,
                                context={
                                    'me': self
                                })


class CradminJavascript(AbstractJsComponent):
    @classmethod
    def get_component_id(cls):
        return 'django_cradmin_javascript'

    def get_sourceurls(self):
        return [
            self.get_static_url('django_cradmin/{version}/scripts/django_cradmin.js'.format(
                version=django_cradmin.__version__
            ))
        ]



from __future__ import unicode_literals
from builtins import object
import collections


class CrAdminInstanceRegistry(object):
    def __init__(self):
        self._registry = collections.OrderedDict()

    def get_instanceclass_by_id(self, id):
        """
        Get a CrAdmin instance class by ID (by :obj:`django_cradmin.crinstance.BaseCrAdminInstance.id`).
        """
        return self._registry[id]

    def get_instance_by_id(self, id, request):
        """
        Shortcut for ``get_instanceclass_by_id(id)(request)``.
        """
        return self.get_instanceclass_by_id(id)(request)

    def add(self, cradmin_instance_class):
        """
        Add a CrAdmin instance to the registry.

        Parameters:
            cradmin_instance_class: A subclass of :class:`django_cradmin.crinstance.BaseCrAdminInstance`.
        """
        if cradmin_instance_class.id not in self._registry:
            self._registry[cradmin_instance_class.id] = cradmin_instance_class


cradmin_instance_registry = CrAdminInstanceRegistry()



from __future__ import unicode_literals

from django.contrib import admin
from django.template import defaultfilters

from django_cradmin.apps.cradmin_imagearchive.models import ArchiveImage


class ArchiveImageAdmin(admin.ModelAdmin):
    list_display = [
        'id',
        'name',
        'description',
        'role',
        'image_width',
        'image_height',
        'get_size',
        'get_url',
    ]
    list_display_links = [
        'id',
        'name',
    ]

    search_fields = [
        'id',
        'name',
        'description',
    ]

    readonly_fields = [
        'image_width',
        'image_height',
        'file_extension',
    ]

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super(ArchiveImageAdmin, self).get_queryset(request)
        else:
            return ArchiveImage.objects.none()

    def get_size(self, obj):
        if obj.file_size:
            return defaultfilters.filesizeformat(obj.file_size)
        else:
            return None
    get_size.short_description = 'Size'
    get_size.admin_order_field = 'file_size'

    def get_url(self, obj):
        return '<a href="{url}" target="_blank">{label}</a>'.format(
            label='Show image',
            url=obj.image.url
        )
    get_url.short_description = 'Open'
    get_url.allow_tags = True


admin.site.register(ArchiveImage, ArchiveImageAdmin)



import calendar

import datetime
from django.utils import timezone
from django.utils.translation import ugettext_lazy, pgettext_lazy

from django_cradmin.viewhelpers.listfilter.base.abstractfilter import AbstractFilter


class AbstractSelectFilter(AbstractFilter):
    """
    Abstract base class for any select filter.

    You only have to override :meth:`~AbstractSelectFilter.get_choices` and
    :meth:`~AbstractCheckboxFilter.filter`.
    """
    template_name = 'django_cradmin/viewhelpers/listfilter/django/single/select/base.django.html'

    def get_choices(self):
        """
        Get choices as a list of of ``(value, label)`` pairs.
        """
        raise NotImplementedError()

    def get_default_is_selected_index(self, choicesdata):
        """
        Get the index of the default selected item in choicesdata.

        Called by :meth:`.get_choicesdata` when no choice matches
        the current cleaned value.

        See the docs for ``is_selected`` in :meth:`.get_choicesdata`
        for more details.

        Parameters:
            choicesdata (list): Same format as the list returned by :meth:`.get_choicesdata`.
        """

        return 0

    def __make_choicesdata_list(self, choices, selected_value):
        selected_value = self.get_cleaned_value()
        choicesdata = []
        found_selected_value = False
        for value, label in choices:
            if isinstance(label, (tuple, list)):
                optgroup_label = value
                optgroup_choices = label
                child_choicesdata, found_selected_value_in_child = self.__make_choicesdata_list(
                    choices=optgroup_choices, selected_value=selected_value)
                if found_selected_value_in_child:
                    found_selected_value = True
                choicesdata.append({
                    'is_optgroup': True,
                    'label': optgroup_label,
                    'child_choicesdata': child_choicesdata
                })
            else:
                is_selected = value == selected_value
                if is_selected:
                    found_selected_value = True
                url = self.build_set_values_url(values=[value])
                choicesdata.append({
                    'url': url,
                    'label': label,
                    'is_selected': is_selected
                })
        return choicesdata, found_selected_value

    def get_choicesdata(self):
        """
        Get choices data as a list of dicts.

        This uses :meth:`.get_choices`, and turns it into
        a list of dicts where each dict has:

        - ``url``: The URL to load when making a choice.
          This is generated using
          :meth:`~django_cradmin.viewhelpers.listfilter.base.AbstractFilter.build_set_values_url`
          with the value from :meth:`.get_choices` as input.
        - ``label``: The label to show (taken directly from :meth:`.get_choices`.
        - ``is_selected`` (boolean): If the current value matches the value of a choice,
          that choice will be selected. If none of them matches, the first choice is selected.
          You can change this behavior by overriding :meth:`.get_default_is_selected_index`.

        Returns:
            A tuple with the list of dicts described above and the selected label.
        """
        # selected_value = self.get_cleaned_value()
        # choicesdata = []
        # found_selected_value = False
        # for value, label in self.get_choices():
        #     is_selected = value == selected_value
        #     if is_selected:
        #         found_selected_value = True
        #     url = self.build_set_values_url(values=[value])
        #     choicesdata.append({
        #         'url': url,
        #         'label': label,
        #         'is_selected': is_selected
        #     })
        choicesdata, found_selected_value = self.__make_choicesdata_list(
            choices=self.get_choices(),
            selected_value=self.get_cleaned_value())
        if not found_selected_value and len(choicesdata) > 0:
            selected_index = self.get_default_is_selected_index(choicesdata=choicesdata)
            choicesdata[selected_index]['is_selected'] = True
        return choicesdata

    def get_context_data(self, request=None):
        context = super(AbstractSelectFilter, self).get_context_data(request=None)
        context['choicesdata'] = self.get_choicesdata()
        return context


class AbstractBoolean(AbstractSelectFilter):
    """
    Abstract boolean filter.

    Subclasses should only need to implement
    :meth:`django_cradmin.viewhelpers.listfilter.basefilters.single.select.AbstractSelectFilter.filter`.

    See :class:`django_cradmin.viewhelpers.listfilter.django.single.select.Boolean`
    for a Django ORM implementation.
    """
    def get_do_not_apply_label(self):
        """
        Returns the label for the default "do not apply this filter"
        option. Defaults to empty string.
        """
        return ''

    def get_true_label(self):
        """
        Get the label for the ``True`` option. Defaults to ``Yes`` (translatable).
        """
        return ugettext_lazy('Yes')

    def get_false_label(self):
        """
        Get the label for the ``False`` option. Defaults to ``No`` (translatable).
        """
        return ugettext_lazy('No')

    def get_choices(self):
        return [
            ('', self.get_do_not_apply_label()),
            ('true', self.get_true_label()),
            ('false', self.get_false_label()),
        ]


class AbstractDateTime(AbstractSelectFilter):
    """
    Abstract DateTime filter.

    Subclasses should only need to implement
    :meth:`~.DateTime.filter_datetime_range`.

    See :class:`django_cradmin.viewhelpers.listfilter.django.single.select.DateTime`
    for a Django ORM implementation.
    """
    def get_do_not_apply_label(self):
        """
        Returns the label for the default "do not apply this filter"
        option. Defaults to empty string.
        """
        return ''

    def get_today_label(self):
        """
        Get the label for the "today" option.

        Defaults to ``Today`` (translatable).
        """
        return ugettext_lazy('Today')

    def get_yesterday_label(self):
        """
        Get the label for the "today" option.

        Defaults to ``Today`` (translatable).
        """
        return ugettext_lazy('Yesterday')

    def get_last_seven_days_label(self):
        """
        Get the label for the "today" option.

        Defaults to ``Today`` (translatable).
        """
        return ugettext_lazy('Last seven days')

    def get_this_week_label(self):
        """
        Get the label for the "this_week" option.

        Defaults to ``This week`` (translatable).
        """
        return ugettext_lazy('This week')

    def get_this_month_label(self):
        """
        Get the label for the "this_month" option.

        Defaults to ``This month`` (translatable).
        """
        return ugettext_lazy('This month')

    def get_this_year_label(self):
        """
        Get the label for the "this_year" option.

        Defaults to ``This year`` (translatable).
        """
        return ugettext_lazy('This year')

    def null_enabled(self):
        """
        Override this if you want ``null`` to be a valid option.
        """
        return False

    def get_is_null_label(self):
        """
        Get the label for the ``null`` option.

        Only used if :meth:`.null_enabled` is overridden to return ``True``.

        Defaults to ``Has no value`` (translatable).
        """
        return pgettext_lazy('listfilter AbstractDateTime', 'Has no value')

    def get_is_not_null_label(self):
        """
        Get the label for the ``not null`` option.

        Only used if :meth:`.null_enabled` is overridden to return ``True``.

        Defaults to ``Has value`` (translatable).
        """
        return pgettext_lazy('listfilter AbstractDateTime', 'Has value')

    def get_choices(self):
        choices = [
            ('', self.get_do_not_apply_label()),
            ('today', self.get_today_label()),
            ('yesterday', self.get_yesterday_label()),
            ('last_seven_days', self.get_last_seven_days_label()),
            ('this_week', self.get_this_week_label()),
            ('this_month', self.get_this_month_label()),
            ('this_year', self.get_this_year_label()),
        ]
        if self.null_enabled():
            choices.insert(1, ('is_not_null', self.get_is_not_null_label()))
            choices.insert(1, ('is_null', self.get_is_null_label()))
        return choices

    def filter_datetime_range(self, queryobject, start_datetime, end_datetime):
        """
        Filter out all items ``>= start_datetime`` and ``< end_datetime``.

        Parameters:
            queryobject: See :meth:`~django_cradmin.viewhelpers.listfilter.base.abstractfilter.AbstractFilter.filter`.
            start_datetime (datetime.datetime): Start datetime.
            end_datetime (datetime.datetime): End datetime.
        """
        raise NotImplementedError()

    def filter_within_date(self, queryobject, dateobject):
        next_day_dateobject = dateobject + datetime.timedelta(days=1)
        day_start = datetime.datetime.combine(dateobject, datetime.time())
        day_end = datetime.datetime.combine(next_day_dateobject, datetime.time())
        return self.filter_datetime_range(queryobject=queryobject,
                                          start_datetime=day_start,
                                          end_datetime=day_end)

    def filter_today(self, queryobject):
        today = timezone.now().date()
        return self.filter_within_date(queryobject=queryobject,
                                       dateobject=today)

    def filter_yesterday(self, queryobject):
        yesterday = timezone.now().date() - datetime.timedelta(days=1)
        return self.filter_within_date(queryobject=queryobject,
                                       dateobject=yesterday)

    def filter_last_seven_days(self, queryobject):
        now = timezone.now()
        today = now.date()
        seven_days_ago = today - datetime.timedelta(days=7)
        seven_days_ago_day_start = datetime.datetime.combine(seven_days_ago, datetime.time())
        return self.filter_datetime_range(queryobject=queryobject,
                                          start_datetime=seven_days_ago_day_start,
                                          end_datetime=now)

    def filter_this_week(self, queryobject):
        today = timezone.now().date()
        first_date_of_week = today - datetime.timedelta(days=today.weekday())
        first_date_of_next_of_week = first_date_of_week + datetime.timedelta(days=7)
        start_of_week = datetime.datetime.combine(first_date_of_week, datetime.time())
        end_of_week = datetime.datetime.combine(first_date_of_next_of_week, datetime.time())
        return self.filter_datetime_range(queryobject=queryobject,
                                          start_datetime=start_of_week,
                                          end_datetime=end_of_week)

    def filter_this_month(self, queryobject):
        today = timezone.now().date()
        first_date_of_month = today.replace(day=1)
        days_in_month = calendar.monthrange(today.year, today.month)[1]
        first_date_of_next_month = first_date_of_month + datetime.timedelta(days=days_in_month)
        start_of_month = datetime.datetime.combine(first_date_of_month, datetime.time())
        end_of_month = datetime.datetime.combine(first_date_of_next_month, datetime.time())
        return self.filter_datetime_range(queryobject=queryobject,
                                          start_datetime=start_of_month,
                                          end_datetime=end_of_month)

    def filter_this_year(self, queryobject):
        today = timezone.now().date()
        first_date_of_year = today.replace(month=1, day=1)
        first_date_of_next_year = first_date_of_year.replace(year=today.year + 1)
        start_of_year = datetime.datetime.combine(first_date_of_year, datetime.time())
        end_of_year = datetime.datetime.combine(first_date_of_next_year, datetime.time())
        return self.filter_datetime_range(queryobject=queryobject,
                                          start_datetime=start_of_year,
                                          end_datetime=end_of_year)

    def filter_is_null(self, queryobject):
        """
        Filter so that only values that are NULL is included.

        Must be overridden in subclasses.
        """
        raise NotImplementedError()

    def filter_is_not_null(self, queryobject):
        """
        Filter so that only values that are not NULL is included.

        Must be overridden in subclasses.
        """
        raise NotImplementedError()

    def filter(self, queryobject):
        cleaned_value = self.get_cleaned_value()
        if cleaned_value == 'today':
            return self.filter_today(queryobject=queryobject)
        elif cleaned_value == 'yesterday':
            return self.filter_yesterday(queryobject=queryobject)
        elif cleaned_value == 'last_seven_days':
            return self.filter_last_seven_days(queryobject=queryobject)
        elif cleaned_value == 'this_week':
            return self.filter_this_week(queryobject=queryobject)
        elif cleaned_value == 'this_month':
            return self.filter_this_month(queryobject=queryobject)
        elif cleaned_value == 'this_year':
            return self.filter_this_year(queryobject=queryobject)
        elif cleaned_value == 'is_null' and self.null_enabled():
            return self.filter_is_null(queryobject=queryobject)
        elif cleaned_value == 'is_not_null' and self.null_enabled():
            return self.filter_is_not_null(queryobject=queryobject)
        return queryobject


class AbstractOrderBy(AbstractSelectFilter):
    """
    Abstract order by filter --- lets the user select how the queryobject should be ordered.

    Subclasses should only need to implement
    :meth:`django_cradmin.viewhelpers.listfilter.basefilters.single.select.AbstractSelectFilter.filter`.

    See :class:`django_cradmin.viewhelpers.listfilter.django.single.select.AbstractOrderBy`
    for a Django ORM implementation.
    """
    def __init__(self, *args, **kwargs):
        super(AbstractOrderBy, self).__init__(*args, **kwargs)
        self.ordering_options = self.get_ordering_options()
        self.ordering_options_dict = self.__make_ordering_options_dict(self.ordering_options)

    def __make_ordering_options_dict(self, ordering_options):
        ordering_options_dict = {}
        for value, option in ordering_options:
            if isinstance(option, (list, tuple)):
                optgroup_options_list = option
                ordering_options_dict.update(
                    self.__make_ordering_options_dict(optgroup_options_list))
            else:
                ordering_options_dict[value] = option
        return ordering_options_dict

    def get_ordering_options(self):
        """
        Get the ordering options - must be overridden in subclasses.

        You must override this instead of ``get_choices()``.

        Should return a list of ``(value, options)`` pairs where
        ``value`` is the same as in :meth:`.AbstractSelectFilter.get_choices`
        and ``options`` is a dict with:

        - ``label``: The label for the ordering option (shown to the user).
        - ``order_by``: A list with the same format as ``*args`` for
            ``QuerySet.order_by()``.

        Examples:

            Lets say we have something with ``title`` and ``publishing_time``,
            and we want to order by ``publishing_time`` descending by default::

                def get_ordering_options(self):
                    return [
                        ('', {
                            'label': 'Publishing time (newest first)',
                            'order_by': ['-publishing_time'],
                        }),
                        ('publishing_time_asc', {
                            'label': 'Publishing time (oldest first)',
                            'order_by': ['publishing_time'],
                        }),
                        ('title', {
                            'label': 'Title',
                            'order_by': ['title'],
                        }),
                    ]

        """
        raise NotImplementedError()

    def __make_choices(self, ordering_options_list):
        choices = []
        for value, option in ordering_options_list:
            if isinstance(option, (list, tuple)):
                optgroup_label = value
                optgroup_options_list = option
                choices.append((optgroup_label, self.__make_choices(optgroup_options_list)))
            else:
                choices.append((value, option['label']))
        return choices

    def get_choices(self):
        return self.__make_choices(self.ordering_options)
